'''
Simple wrapper for tests
'''


import sys, unittest

class Webtestcase(unittest.TestCase):
    '''
    Simple test outline and utilities
    '''
    def __init__(self, testMethod, name = "", dieOnError=True, logname="", *args):
        if (testMethod is None):
            raise Exception("Test Method must be defined")
        else:
            self.testMethod = testMethod
        if (name != ""):
            self.name = str(name)
        else:
            name = str(testMethod.__name__)
        self.dieOnError = dieOnError
        self.log = None
        self.logname = str(logname)
        self.savestdout = sys.stdout
        
        
        
    def redirectToFile(self,filename):
            self.log = open(filename, 'w')
            if (self.log is None):
                raise Exception("Could not create log file")
            sys.stdout =  self.log
            
            
        
        
    def runTest(self):
        try:
            print "TEST: "+self.name+ "STARTING..."
            if (self.logname != ""):    
                self.setupLog(self.logname)
            self.testMethod()
            print "TEST: "+self.name+" PASSED"
            self.printPassed()
        except Exception, e:
            print "TEST: "+self.name+ " FAILED"
            self.printFailed()
            if (self.dieOnError):
                self.printClosing()
                raise e
        finally:
            sys.stdout = self.savestdout
            
            
        
    
        
    def printClosing(self):
        print "\n"
        print ''' 
     _____ _           _ 
    /  __ \ |         (_)
    | /  \/ | ___  ___ _ _ __   __ _ 
    | |   | |/ _ \/ __| | '_ \ / _` |
    | \__/\ | (_) \__ \ | | | | (_| |
     \____/_|\___/|___/_|_| |_|\__, |
                                __/ |
                               |___/ 
    
    '''
        print "\n"



    def printPassed(self):
        print "\n"
        print ''' 
    ______  ___  _____ _____ ___________  
    | ___ \/ _ \/  ___/  ___|  ___|  _  \ 
    | |_/ / /_\ \ `--.\ `--.| |__ | | | | 
    |  __/|  _  |`--. \`--. \  __|| | | | 
    | |   | | | /\__/ /\__/ / |___| |/ /  
    \_|   \_| |_|____/\____/\____/|___/   
    
    '''
        print "\n"
    
    
    def printFailed(self):
        print "\n"
        print '''
    ______ ___  _____ _      ___________ 
    |  ___/ _ \|_   _| |    |  ___|  _  \ 
    | |_ / /_\ \ | | | |    | |__ | | | | 
    |  _||  _  | | | | |    |  __|| | | | 
    | |  | | | |_| |_| |____| |___| |/ /  
    \_|  \_| |_/\___/\_____/\____/|___/  
                                         
    '''
        print "\n"
        
        
        
        


# Software License Agreement (BSD License)
#
# Copyright (c) 2009-2011, Eucalyptus Systems, Inc.
# All rights reserved.
#
# Redistribution and use of this software in source and binary forms, with or
# without modification, are permitted provided that the following conditions
# are met:
#
#   Redistributions of source code must retain the above
#   copyright notice, this list of conditions and the
#   following disclaimer.
#
#   Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the
#   following disclaimer in the documentation and/or other
#   materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# Author: Vic Iglesias vic.iglesias@eucalyptus.com
#
#!/usr/bin/python
import os
import re
import time
import random
from eucaops import EC2ops, Eucaops
from eutester.euinstance import EuInstance
from eutester.eutestcase import EutesterTestCase
from eutester.machine import Machine

class ImageCreator(EutesterTestCase):
    def __init__(self):
        extra_args = ['--size', '--repo-url', '--packages', '--user-data']
        self.setuptestcase()
        self.setup_parser()
        if extra_args:
            for arg in extra_args:
                self.parser.add_argument(arg)
        self.get_args()
        # Setup basic eutester object
        if self.args.region:
            self.tester = EC2ops( credpath=self.args.credpath, region=self.args.region)
        else:
            self.tester = Eucaops(config_file=self.args.config, password=self.args.password, credpath=self.args.credpath)
        self.tester.poll_count = 120

        ### Add and authorize a group for the instance
        self.group = self.tester.add_group(group_name="group-" + str(time.time()))
        self.tester.authorize_group_by_name(group_name=self.group.name )
        self.tester.authorize_group_by_name(group_name=self.group.name, port=-1, protocol="icmp" )
        ### Generate a keypair for the instance
        self.keypair = self.tester.add_keypair( "keypair-" + str(time.time()))
        self.keypath = '%s/%s.pem' % (os.curdir, self.keypair.name)
        if not self.args.emi:
            raise Exception("Must pass base image id to use as parameter --emi")
        self.image =  self.tester.get_emi(self.args.emi)
        self.address = None
        self.volume = None
        self.private_addressing = False
        zones = self.tester.ec2.get_all_zones()
        self.zone = random.choice(zones).name
        self.reservation = None

    def clean_method(self):
        if self.reservation:
            self.tester.terminate_instances(self.reservation)
            self.reservation = None
        if self.volume:
            self.tester.delete_volume(self.volume)

    def CreateImage(self, zone= None):
        '''Register a BFEBS snapshot'''
        if zone is None:
            zone = self.zone
        user_data = open(self.args.user_data, mode="r").read()
        self.reservation = self.tester.run_instance(image=self.args.emi,keypair=self.keypair.name, group=self.group.name,
                                                    zone=zone, user_data=user_data)
        for instance in self.reservation.instances:
            if instance.root_device_type == "ebs":
                self.CreateEBS(instance)
            else:
                self.CreateInstanceStore(instance)
        self.tester.terminate_instances(self.reservation)
        self.reservation = None

    def get_machine(self, instance):
        assert isinstance(instance, EuInstance)
        distro = instance.sys('head -1 /etc/issue')[0]
        if re.search("CentOS", distro):
            return Machine(instance.ip_address, keypath=self.keypath, distro="CENTOS", distro_ver="6")
        elif re.search("Ubuntu", distro):
            return Machine(instance.ip_address, keypath=self.keypath, distro="UBUNTU", distro_ver="PRECISE")
        raise Exception("Unable to find supported distro on image")

    def CreateEBS(self, instance):
        machine = self.get_machine(instance)
        if hasattr(self.args, 'repo'):
            machine.package_manager.add_repo(self.args.repo_url)
        if hasattr(self.args, 'packages'):
            machine.package_manager.install(self.args.packages)
        volume = self.tester.get_volumes(attached_instance=instance.id)[0]
        snapshot = self.tester.create_snapshot(volume.id)
        self.tester.register_snapshot(snapshot)

    def CreateInstanceStore(self, instance):
        machine = self.get_machine(instance)
        if hasattr(self.args, 'repo'):
            machine.package_manager.add_repo(self.args.repo_url)
        if hasattr(self.args, 'packages'):
            machine.package_manager.install(self.args.packages)
        mount_point = "/mnt"
        instance.sys("mfks.ext3 -F /dev/" + instance.rootfs_device + "2" )
        instance.sys("mount " + "/dev/" + instance.rootfs_device + "2 " + mount_point )

        image_file_name = "server.img"
        remote_image_file = mount_point + "/" + image_file_name
        instance.sys("dd bs=1M if=/dev/" + instance.rootfs_device + "1 of=" + remote_image_file, timeout=600)
        machine.sftp.get(remote_image_file, image_file_name)

    def find_filesystem(self, machine, block_device):
        for device in machine.sys('ls -1 ' + block_device + "*"):
            if machine.found('file -s ' + device, "filesystem"):
                return device
        raise Exception("Unable to find a filesystem on block device:" + block_device)

if __name__ == "__main__":
    testcase = ImageCreator()
    ### Either use the list of tests passed from config/command line to determine what subset of tests to run
    list = [ "CreateImage"]
    ### Convert test suite methods to EutesterUnitTest objects
    unit_list = [ ]
    for test in list:
        unit_list.append( testcase.create_testunit_by_name(test) )
        ### Run the EutesterUnitTest objects

    result = testcase.run_test_case_list(unit_list,clean_on_exit=True)
    exit(result)



'''
Created on Nov 11, 2011

@author: clarkmatthew
'''
import random,string, time
#import eulogger
#logger = eulogger.Eulogger()
#log = logger.log


##########################################################################################            
##########################################################################################

class Euwebuser(object):
    
    
    def __init__ (self, 
                  account="", 
                  user="", 
                  passwd="", 
                  credpath="", 
                  email = "", 
                  prefix="",
                  path="",
                  uid="",
                  enabled="",
                  regstatus="",
                  makerandom=True):
        
            print "Init euwebuser...."
            print("creating user...")
            #If the ID is not provided use the username, if the user is not provided create a random one
            self.account = account
            self.user = user
            self.passwd = passwd
            self.credpath = credpath
            self.email = email
            self.prefix = prefix
            self.path = path
            self.id = uid
            self.enabled = enabled
            self.status = regstatus
            self.rid = "NA"
            
            if (makerandom == True):
                try:
                    rand = random.Random()
                    time.sleep(1)# just to make sure seed is unique
                    self.rid = str(rand.randint(1, 100000000))
                except Exception, e:
                    print "Couldn't gen random id use time.clock instead:" + str(e)
                    rid = str(time.clock())
                
                #UID should be created by the CLC not the user
                self.id = uid
                    
                if (account != ""):
                    if (account == "random"):
                        self.account = prefix+"account-"+self.rid
                    else:
                        self.account = account
                else:
                    self.account = prefix+"account-"+self.rid
                
                if (user):
                    if (user == "random"):
                        self.user = prefix+self.rid
                    else:
                        self.user = user
                else:
                    self.user = "admin"
                
                if (passwd):
                    self.passwd = passwd
                else:
                    self.passwd = "passwd"+self.rid 
                
                if (credpath):
                    self.credpath = credpath
                else:
                    self.credpath = self.user+"_dir"
                    
                if (email):
                    self.email = email
                else:
                    self.email = self.user+'@'+self.account
                
            
                             
    def printUser(self):
        print("+++++++++++++PRINT USER+++++++++++++++++++++")
        print("id:"+self.id)
        print("account:"+self.account)
        print("user:"+self.user)
        print("passwd:"+self.passwd)
        print("credpath:"+self.credpath)
        print("email: " +self.email)
        print("enabled:"+self.enabled)
        print("regstat:"+self.status)
        print("++++++++++++++++++++++++++++++++++++++++++++")
        
        
        
        
        
        
        


# Software License Agreement (BSD License)
#
# Copyright (c) 2009-2011, Eucalyptus Systems, Inc.
# All rights reserved.
#
# Redistribution and use of this software in source and binary forms, with or
# without modification, are permitted provided that the following conditions
# are met:
#
#   Redistributions of source code must retain the above
#   copyright notice, this list of conditions and the
#   following disclaimer.
#
#   Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the
#   following disclaimer in the documentation and/or other
#   materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# Author: vic.iglesias@eucalyptus.com
'''
Created on Apr 25, 2012
@author: vic.iglesias@eucalyptus.com
'''

import pickle
import ConfigParser
import random 
import os
import string
class EuPopulator(object):
    '''
    This class is intended to read in a config file using ConfigParser, then create resources based on the config file
    It is also able to serialize the current resources in a cloud into a file, read in that file and verify that the resources still
    exist in the same way. Serialization is done with Pickle. 
    '''

    def __init__(self, eucaops, config_file = None):
        '''
        Constructor for EuPopulator:
            eutester         A eutester object that we can use for connections
            config_file      Filename of the config describing what resources to populate
        '''
        self.tester = eucaops
        if config_file is None:
            self.defaults = ConfigParser.SafeConfigParser()
            ### Global 
            self.defaults.add_section('global')
            partition = self.tester.ec2.get_all_zones()[0].name
            self.defaults.set('global', 'partition', partition)
            ### Volume 
            self.defaults.add_section('volumes')
            self.defaults.set('volumes', 'count', '2')
            self.defaults.set('volumes', 'min_size', '1')
            self.defaults.set('volumes', 'max_size', '3')
            
            ### Addresses
            self.defaults.add_section('addresses')
            self.defaults.set('addresses', 'count', '2')
            
            ### Snapshot 
            self.defaults.add_section('snapshots')
            self.defaults.set('snapshots', 'count', '2')
            self.defaults.set('snapshots', 'min_size', '1')
            self.defaults.set('snapshots', 'max_size', '3')
            self.defaults.set('snapshots', 'delete_volumes', 'false')
            
            ### Keypair 
            self.defaults.add_section('keypairs')
            self.defaults.set('keypairs', 'count', '2')
            self.defaults.set('keypairs', 'prefix', self.prefix_generator())
            
            ### Groups 
            self.defaults.add_section('security_groups')
            self.defaults.set('security_groups', 'count', '2')
            self.defaults.set('security_groups', 'prefix', self.prefix_generator())
            self.defaults.set('security_groups', 'port_min', '22')
            self.defaults.set('security_groups', 'port_max', '80')
            self.defaults.set('security_groups', 'protocol', 'tcp')
            self.defaults.set('security_groups', 'cidr', '0.0.0.0/0')
            
            ### Buckets
            self.defaults.add_section('buckets')
            self.defaults.set('buckets', 'count', '2')
            self.defaults.set('buckets', 'prefix', self.prefix_generator())
            self.defaults.set('buckets', 'add_keys', 'true')
            self.defaults.set('buckets', 'key_count', '2')
            self.defaults.set('buckets', 'key_prefix', self.prefix_generator())
            self.config = self.defaults

        else:
            config = ConfigParser.SafeConfigParser()
            config.read(config_file)
            self.config = config
        
        if self.config:
            with open('populate.cfg', 'wb') as configfile:
                self.config.write(configfile)
        else:
            raise Exception("Do not have a valid config file to use")
        
    def populate(self):
        '''
        Takes the config and creates each of the resources based on their parameters
        '''
        for section in self.config.sections():
            if section is not "global":
                try:
                    creation_method = getattr(EuPopulator, section)
                    creation_method(self)
                    print "Successfully created: " + section
                except Exception, e:
                    print "Unable to bind to resource creation method for: " + section + "\n Because of " + str(e)
                
    def volumes(self):
        print "Creating volumes"
        partition = self.config.get("global","partition")
        vol_count = self.config.getint("volumes", "count")
        min_size = self.config.getint("volumes", "min_size")
        max_size = self.config.getint("volumes", "max_size")
        
        for i in xrange(vol_count):
            size = random.randint(min_size, max_size)
            self.tester.create_volume(partition, size)
        
    def addresses(self):
        print "Allocating addreses"
        addr_count =  self.config.getint("addresses", "count")
        for i in xrange(vol_count):
            self.tester.allocate_address()
        
    def snapshots(self):
        print "Creating snapshots"
        partition = self.config.get("global","partition")
        snap_count = self.config.getint("snapshots", "count")
        min_size = self.config.getint("snapshots", "min_size")
        max_size = self.config.getint("snapshots", "max_size")
        delete_volumes = self.config.getboolean('snapshots', 'delete_volumes')
        
        for i in xrange(snap_count):
            size = random.randint(min_size, max_size)
            volume = self.tester.create_volume(partition, size)
            self.tester.create_snapshot(volume.id)
            if delete_volumes:
                self.tester.delete_volume(volume)
    
    def keypairs(self):
        print "Creating keypairs"
        key_count = self.config.getint("keypairs", "count")
        prefix = self.config.get("keypairs", "prefix")
        
        for i in xrange(key_count):
            self.tester.add_keypair(prefix + str(i))
        
    def security_groups(self):
        print "Creating security_groups"
        sg_count = self.config.getint("security_groups", "count")
        prefix = self.config.get("security_groups", "prefix")
        port_min = self.config.getint("security_groups", "port_min")
        port_max = self.config.getint("security_groups", "port_max")
        cidr = self.config.get("security_groups", "cidr")
        protocol = self.config.get("security_groups", "protocol")
        for i in xrange(sg_count):
            group = self.tester.add_group(prefix + str(i))
            self.tester.authorize_group(group, port=random.randint(port_min, port_max), protocol=protocol, cidr_ip=cidr)
    
    def buckets(self):
        print "Creating buckets"
        bucket_count = self.config.getint('buckets', 'count')
        bucket_prefix = self.config.get('buckets', 'prefix')
        add_keys = self.config.getboolean('buckets', 'add_keys')
        key_count = self.config.getint('buckets', 'key_count')
        key_prefix = self.config.get('buckets', 'key_prefix')
        
        for i in xrange(bucket_count):
            bucket = self.tester.create_bucket(bucket_prefix + str(i))
            if add_keys:
                for i in xrange(key_count):
                    self.tester.upload_object(bucket.name, key_prefix + str(i))
        
    def depopulate(self):
        self.tester.cleanup_artifacts()  
    
    def prefix_generator(self, size=6, chars=string.ascii_lowercase + string.digits):
        return ''.join(random.choice(chars) for x in range(size))
    
    def serialize_resources(self, output_file="cloud_objects.dat"):
        '''
        Takes a snapshot of the current resources in the system and puts them into a data structure, then pickles this struct into a file.
            output_file    Filename to use when outputing the serialized data, if this is None then the byte stream will be returned
        '''
        self.tester.debug("Serializing current resources")
        serialized = open(output_file, 'wb')
        all_resources = self.tester.get_current_resources()
        for type in all_resources:
            for object in all_resources[type]:
                print >>serialized, self.tester.get_all_attributes(object)
            
         
        serialized.close()
        self.tester.debug("Serialized resources can be found at: " + output_file)
        


#!/usr/bin/python
# Software License Agreement (BSD License)
#
# Copyright (c) 2009-2011, Eucalyptus Systems, Inc.
# All rights reserved.
#
# Redistribution and use of this software in source and binary forms, with or
# without modification, are permitted provided that the following conditions
# are met:
#
#   Redistributions of source code must retain the above
#   copyright notice, this list of conditions and the
#   following disclaimer.
#
#   Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the
#   following disclaimer in the documentation and/or other
#   materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# Author: matt.clark@eucalyptus.com

"""
Description:
Set of methods to aid in multipath testing. Main test points are ebs usage during blocking and unblocking network i/o to
 device mapper multipath paths via iptables.
 Node Controller Test points:
    -confirm multiple paths are set on the NC and/or SC in order to run test, else will throw 'SkipTestException' for
     testcase class to handle
    -constant i/o on guest mounted volume with read, write, and checks during mpath failovers
    -attach and detach during different phases of path(s) failing over.
    -attach a volume to a guest while a single path is down, verify read/write direct to device on guest
    -attach a volume while a path is in the process of failing
    -attach a volume with all paths in good state, then detach while single path is down.
    -attach a volume to a guest while a single path is down, verify mounted formated volume i/o, verify detach
    -Multiple volume attach/detach checks with different path up/down states.
Storage Controller Test points:
    -If multiple sanhosts are provided in the SC's storage property, will attempt to test sanhost fail over using iptables
    to block network i/o to a given host and iterate through the hosts. Will attempt to create volumes to verify functionality.
    -If multiple scpaths are provided in the storage property, will attempt to block dev mapper paths with iptables, while
    creating snapshots. Will test blocking paths before and during snapshot creation.
"""

__author__ = 'clarkmatthew'


from eutester.eutestcase import EutesterTestCase, SkipTestException
from eutester.eutestcase import TestColor
from eucaops import ec2ops
#from eutester.euinstance import EuInstance
#from eutester.euvolume import EuVolume
#from eutester.eusnapshot import EuSnapshot
from eutester.sshconnection import SshCbReturn
from eutester.euproperties import Euproperty_Type, EupropertyNotFoundException
from testcases.cloud_user.ebs.path_controller import Path_Controller
from eucaops import Eucaops
import eutester
import time
import copy
import os
import sys
import signal
import re
import curses

class Mpath_Suite(EutesterTestCase):
    stopped_status = 'STOPPED'
    def __init__(self, tester=None, path_controllers=None, **kwargs):
        #### Pre-conditions
        self.setuptestcase()
        self.setup_parser(testname='Multipath_suite')

        self.parser.add_argument('--local_path_to_nc_script',
                                 dest='io_script_path',
                                 help='Path to NC IO generator script if not in local dir, default: "vm_read_write_vol.py"',
                                 default='vm_read_write_vol.py')
        self.parser.add_argument('--size',
                                 type=int,
                                 help='Size in GB for created volumes, default:1',
                                 default=1)
        self.parser.add_argument('--instance_id',
                                 help='Instance id of an existing and running instance, default:None',
                                 default=None)
        self.parser.add_argument('--volume_id',
                                 help='Volume id of an existing and volume, default:None',
                                 default=None)
        self.parser.add_argument('--inter_io_timeout',
                                 type=int,
                                 help='Max time in seconds to wait for remote ssh command to update before failing, default:30',
                                 default=30)
        self.parser.add_argument('--timepergig',
                                 type=int,
                                 help='Time allowed per gig size of volume during volume creation, default:120',
                                 default=120)
        self.parser.add_argument('--deletetimeout',
                                 type=int,
                                 help='Time allowed for volume to transition from deleting to deleted, default:120',
                                 default=120)
        self.parser.add_argument('--cycle_paths',
                                 action='store_true', default=True,
                                 help='Boolean used to cycle paths during basic run instance monitor io')
        self.parser.add_argument('--cycle_path_interval',
                                 type=int,
                                 help='Number of seconds between cycling NC paths, default: 15 seconds',
                                 default=15)
        self.parser.add_argument('--max_path_iterations',
                                 type=int,
                                 help='Number of times to iterate over all NC paths, default:2',
                                 default=2)
        self.parser.add_argument('--timed_test_period',
                                 type=int,
                                 help='Number of seconds for timed tests to run instance monitor, default:30',
                                 default=30)

        self.parser.add_argument('--path_recovery_interval',
                                 type=int,
                                 help='Number of seconds to allow a down path to recover before cycling to next, default:30',
                                 default=30)

        self.parser.add_argument('--run_suite',
                                action='store_true', default=True,
                                help='Boolean, will run all test methods in testsuite()')

        self.tester = tester
        self.get_args()
        # Allow __init__ to get args from __init__'s kwargs or through command line parser...
        for kw in kwargs:
            print 'Setting kwarg:'+str(kw)+" to "+str(kwargs[kw])
            self.set_arg(kw ,kwargs[kw])
        self.show_args()
        #if self.args.config:
        #    setattr(self.args, 'config_file',self.args.config)
        # Setup basic eutester object
        if not self.tester:
            try:
                self.tester = self.do_with_args(Eucaops)
            except Exception, e:
                raise Exception('Couldnt create Eucaops tester object, make sure credpath, '
                                'or config_file and password was provided, err:' + str(e))

        self.test_tag = 'mpath_suite'
        #replace default eutester debugger with eutestcase's for more verbosity...
        self.tester.debug = lambda msg: self.debug(msg, traceback=2, linebyline=False)
        self.reservation = None
        self.instance = None
        self.volume = None
        self.size = int(self.args.size)
        self.inter_io_timeout = int(self.args.inter_io_timeout)
        ### Add and authorize a group for the instance
        if self.args.zone:
            self.zone = str(self.args.zone)
        else:
            self.zone = 'PARTI00'
        self.groupname = 'jenkins'
        self.group = self.tester.add_group(self.groupname)
        self.tester.authorize_group(self.group)
        self.tester.authorize_group(self.group, protocol='icmp',port='-1')
        ### Generate a keypair for the instance
        try:
            keys = self.tester.get_all_current_local_keys()
            if keys:
                self.keypair = keys[0]
            else:
                self.keypair = self.tester.add_keypair('mpathtestinstancekey'+str(time.time()))
        except Exception, ke:
            raise Exception("Failed to find/create a keypair, error:" + str(ke))


        ### Get an image to work with
        if self.args.emi:
            self.image = self.tester.get_emi(emi=str(self.args.emi))
        else:
            self.image = self.tester.get_emi(root_device_type="instance-store",not_location='windows')
        if not self.image:
            raise Exception('couldnt find instance store image')
        self.clean_method = self.cleanup

        self.timed_test_period = self.args.timed_test_period
        self.instance = None
        self.volumes = []
        self.remote_script_path = None
        self.longest_wait_period = 0
        self.path_controller = None
        self.cycle_paths =self.args.cycle_paths
        self.current_blocked_path = None
        self.last_blocked_path = None
        self.current_cycle_method = self.cycle_method_loop_over_paths
        self.cycle_path_interval = self.args.cycle_path_interval
        self.path_recovery_interval = self.args.path_recovery_interval
        self.max_path_iterations = self.args.max_path_iterations
        self.path_controllers = path_controllers or []
        self.path_controller = None
        self.stdscr = None
        self.curses_fd = None
        self.testvol_w_checksum = None


    def get_enabled_storage_controller_for_zone(self, zone=None):
        zone = zone or self.zone
        sc_list = self.tester.service_manager.get_all_storage_controllers(partition=zone, state='ENABLED',use_cached_list=False)
        if sc_list:
            sc = sc_list[0]
        else:
            tb = self.tester.get_traceback()
            raise Exception('\n' + str(tb) + 'No enabled storage controllers found in partition:' + str(zone))
        return sc


    def memo_use_multipathing_check(self):
        if self.has_arg('USE_MULTIPATHING') and re.search('YES', self.args.USE_MULTIPATHING,  re.IGNORECASE):
            self.debug('USE_MULTIPATHING flag set in memo field ')
            return True
        else:
            self.debug('USE_MULTIPATHING flag not set in memo field')
            return False


    def zone_has_multiple_nc_paths(self, zone=None):
        zone = zone or self.zone
        try:
            ncpaths_property = self.tester.property_manager.get_property(service_type=Euproperty_Type.storage,partition=zone,name='ncpaths')
            paths = str(ncpaths_property.value).split(',')
        except EupropertyNotFoundException:
            return False
        if len(paths) > 1:
            self.debug('Multiple paths detected on this systems partition:' +str(zone))
            return True
        else:
            self.debug('Multiple paths NOT detected on this systems partition:' +str(zone))
            return False

    def zone_has_multiple_sc_paths(self, zone=None):
        zone = zone or self.zone
        try:
            scpaths_property = self.tester.property_manager.get_property(service_type=Euproperty_Type.storage,partition=zone,name='scpaths')
            paths = str(scpaths_property.value).split(',')
        except EupropertyNotFoundException:
            return False
        if len(paths) > 1:
            self.debug('Multiple SC paths detected on this systems partition:' +str(zone))
            return True
        else:
            self.debug('Multiple SC paths NOT detected on this systems partition:' +str(zone))
            return False

    def zone_has_multiple_sc_hosts(self, zone=None):
        zone = zone or self.zone
        try:
            schost_property = self.tester.property_manager.get_property(service_type=Euproperty_Type.storage,partition=zone,name='sanhost')
            hosts = str(schost_property.value).split(',')
        except EupropertyNotFoundException:
            return False
        if len(hosts) > 1:
            self.debug('Multiple san hosts detected on this systems partition:' +str(zone))
            return True
        else:
            self.debug('Multiple san hosts not detected on this systems partition:' +str(zone))
            return False

    def pre_test_check_should_run_nc_multipath_tests_on_this_system(self,zone=None):
        zone = zone or self.zone
        if self.zone_has_multiple_nc_paths(zone=zone):
            return True
        if self.memo_use_multipathing_check():
            raise Exception('Multipathing enabled in Memo field, but multiple paths not detected in "ncpaths" property')
        raise SkipTestException('Multiple paths not detected, nor was "USE_MULTIPATHING" flag set in config, exiting w/o running tests')


    def pre_test_check_should_run_sc_multipath_tests_on_this_system(self,zone=None):
        zone = zone or self.zone
        if self.zone_has_multiple_sc_paths(zone=zone):
            return True
        if self.memo_use_multipathing_check():
            raise Exception('Multipathing enabled in Memo field, but multiple paths not detected in "scpaths" property')
        raise SkipTestException('Multiple paths not detected, nor was "USE_MULTIPATHING" flag set in config, exiting w/o running tests')

    def pre_test_check_should_run_sc_multihosts_tests_on_this_system(self,zone=None):
        zone = zone or self.zone
        if self.zone_has_multiple_sc_hosts(zone=zone):
            return True
        raise SkipTestException('Multiple san hosts not detected, exiting w/o running tests')





    def create_controller_for_each_node(self):
        node_list =  self.tester.service_manager.get_all_node_controllers()
        for node in node_list:
            add_to_list = True
            for pc in self.path_controllers:
                if pc.host == node.hostname:
                         add_to_list = False
            if add_to_list:
                paths = self.get_nc_paths_by_node(node)
                path_controller = Path_Controller(node=node, sp_ip_list=paths)
                self.path_controllers.append(path_controller)

    def clear_rules_on_all_nodes(self):
        self.status('Clearing eutester rules on all nodes...')
        self.create_controller_for_each_node()
        for pc in self.path_controllers:
            self.status('Clearing all rules on: ' + str(pc.host) )
            pc.clear_all_eutester_rules()




    def cycle_method_loop_over_paths(self,instance=None, clear_retry_interval=2, clear_timeout=30):
        '''
        uses select() in ssh cmd's input to give this method sudo cycles to run
        Loops over paths until stopped or 'path_iterations is reached
        '''
        #self.debug("cycle_method_loop_over_paths...")
        status = 'unknown'
        instance = instance or self.instance
        path_controller = instance.path_controller
        if path_controller.total_path_iterations >= self.max_path_iterations:
            status = 'Max iterations met'
            return
        last_block_time = path_controller.last_block_time
        last_cleared_time = path_controller.last_cleared_time
        last_clear_attempt_time = path_controller.last_clear_attempt_time

        blocked_paths = path_controller.get_blocked_paths()
        now = time.time()

        #See if were in the process of clearing or blocking...
        if last_clear_attempt_time > last_block_time:
            #Clearing...
            #See if we need to keep clearing currently blocked paths...
            if blocked_paths:
                status = 'Waiting for blocked paths to clear'
                if (now - last_clear_attempt_time) > clear_timeout:
                    raise Exception('Could not clear paths within ' + str(clear_timeout) + 'seconds:' + ",".join(blocked_paths))
                if (now - last_clear_attempt_time > clear_retry_interval):
                    #Try to clear all eutester iptables rules
                    path_controller.clear_all_eutester_rules(retry=False)
            elif last_cleared_time and ((now - last_cleared_time) > self.path_recovery_interval):
                status = "Issued block on new path"
                path_controller.block_next_path()
            elif not last_cleared_time:
                status = 'Waiting for iptables to clear all eutester rules'
                path_controller.clear_all_eutester_rules(retry=False)
            else:
                status = "Waiting for recovery interval"
        #We're blocking, see how long it's been and if we need to clear...
        elif (now - last_block_time) > self.cycle_path_interval:
            path_controller.clear_all_eutester_rules(retry=False)
            status = 'Clearing blocked path'
        blocked_paths = "CURRENTLY BLOCKED:" + str(path_controller.get_blocked_string())
        return blocked_paths + ", TEST STATUS:" + status


    def cycle_method_run_guest_io_for_time(self):
        time_run = int(time.time() - self.test_start_time)
        time_remaining = int(self.timed_test_period - time_run)
        if time_remaining > 0:
            status = 'TIME REMAINING IN TEST:' + str(time_remaining)

        else:
            time_remaining = 0
            status = self.stopped_status
        return status



    def cycle_method_loop_over_paths_twice(self):
        self.debug('Looping over each path twice')

    def clear_all_rules_on_controller(self, controller, timeout=60):
        controller.clear_all_eutester_rules(timeout=timeout)

    def cleanup(self, instances=True):
        '''
        Attempts to remove all rules on nodes which were used in test and
        clean up resources created during this test...
        '''
        try:
            for path_controller in self.path_controllers:
                try:
                    host = path_controller.host
                    path_controller.clear_all_eutester_rules(timeout=120)
                except Exception, e:
                    self.debug('Error cleaning up iptables rules on NC:' + str(host) +', Err:'+str(e))
            self.tester.cleanup_artifacts()
        except Exception, e:
            tb = self.tester.get_traceback()
            raise Exception('Cleanupfailed:'+str(e) + "\n" +str(tb))

    def get_test_instance(self):
        instance = self.get_existing_test_instance(instance_id=self.args.instance_id)
        if not instance:
            instance = self.tester.run_image(image=self.image,
                                                   zone=self.zone,
                                                   min=1,
                                                   max=1,
                                                   group=self.group,
                                                   keypair=self.keypair,
                                                   monitor_to_running=True)[0]
        else:
            instance.init_volume_list()
        if not self.test_tag in instance.tags:
            instance.add_tag(self.test_tag)
        self.instance = instance
        self.create_path_controller_for_instance(instance=instance)
        self.path_controller = instance.path_controller
        return instance

    def get_existing_test_instance(self, instance_id=None):
        instances = self.tester.get_connectable_euinstances()
        for instance in instances:
            if instance_id:
                if instance.id == instance_id:
                    return instance
            else:
                if self.test_tag in instance.tags:
                    return instance
        if instance_id:
            raise Exception('Failed to fetch instance from id provided:' +str(self.args.instance_id))
        return None


    def get_nc_paths_by_instance(self,instance=None, iface=False):
        paths = []
        instance = instance or self.instance
        partition = instance.placement
        ncpaths_property = self.tester.property_manager.get_property(service_type=Euproperty_Type.storage,partition=partition,name='ncpaths')
        for path in str(ncpaths_property.value).split(','):
            for part in path.split(':'):
                if re.search('iface', part):
                    if iface:
                        paths.append(part)
                elif not iface:
                    paths.append(part)
        return paths

    def get_nc_paths_by_node(self, node, iface=False):
        paths = []
        partition = node.partition.name
        ncpaths_property = self.tester.property_manager.get_property(service_type=Euproperty_Type.storage,partition=partition,name='ncpaths')
        for path in str(ncpaths_property.value).split(','):
            for part in path.split(':'):
                if re.search('iface', part):
                    if iface:
                        paths.append(part)
                elif not iface:
                    paths.append(part)
        return paths

    def get_sc_sanhosts_by_sc(self, sc):
        partition = sc.partition
        sanhost_property = self.tester.property_manager.get_storage_sanhost_value(partition=partition)
        if not sanhost_property:
            tb = self.tester.get_traceback()
            raise Exception('\n' + str(tb) + 'San host property not found for sc:' + str(sc.hostname))
        return sanhost_property.split(',')

    def get_sc_paths_by_sc(self, sc, iface=False):
        paths = []
        partition = sc.partition
        scpaths_property = self.tester.property_manager.get_property(service_type=Euproperty_Type.storage,partition=partition,name='scpaths')
        for path in str(scpaths_property.value).split(','):
            for part in path.split(':'):
                if re.search('iface', part):
                    if iface:
                        paths.append(part)
                elif not iface:
                    paths.append(part)
        return paths


    def get_node_instance_is_running_on(self, instance=None):
        instance = instance or self.instance
        nodes = self.tester.service_manager.get_all_node_controllers(instance_id=self.instance.id,use_cached_list=False)
        if not nodes:
            raise Exception('Could not find node that instance:'+str(instance.id)+" is running on")
        if len(nodes) > 1:
            nodes_string = ""
            for node in nodes:
                nodes_string += ", " +str(node.hostname)
            raise Exception('Multiple nodes found for instance:'+str(instance.id)+' ' +str(nodes_string))
        node = nodes[0]
        self.debug('Got node:' + str(node.hostname) + ", for instance:" + str(instance.id))
        return node


    def create_path_controller_for_instance(self,instance=None):
        path_controller = None
        instance = instance or self.instance
        node = self.get_node_instance_is_running_on(instance=instance)
        for pc in self.path_controllers:
            if pc.node == node:
                path_controller = pc
                break
        if not path_controller:
            paths = self.get_nc_paths_by_instance(instance=instance)
            path_controller = Path_Controller(node=node, sp_ip_list=paths)
        if not path_controller in self.path_controllers:
            self.path_controllers.append(path_controller)
        instance.path_controller = path_controller
        instance.node = node
        return path_controller


    def get_existing_test_volumes(self, tagkey=None):
        volumes = []
        if self.args.volume_id:
            volumes = self.tester.get_volumes(status='available',volume_id=str(self.args.volume_id))
            if not volumes:
                raise Exception('Failed to fetch volume from id provided:' +str(self.args.volume_id))
        else:
            tagkey = tagkey or self.test_tag
            volumes = self.tester.get_volumes(status='available', filters={'tag-key':str(tagkey)})
        return volumes

    def get_test_volumes(self, count=1):
        '''
        Attempt to retrieve a list of volumes created by this test, or a previous run of this test and in available state.
        If the test can not recycle enough test volumes, it will create new ones so 'count' number of test volumes
        can be returned.

        :param count: int representing the number of test volumes desired.
        '''
        volumes = []
        if count == 1 and self.volume:
            self.volume.update()
            if self.volume.status == 'available':
                return [self.volume]
        volumes = self.get_existing_test_volumes()
        if len(volumes) >= count:
            volumes = volumes[0:count]
        elif len(volumes) < count:
            new_volumes = self.tester.create_volumes(self.zone, size=self.size, count=(count-len(volumes)))
            for volume in new_volumes:
                volume.add_tag(str(self.test_tag))
            volumes.extend(new_volumes)
        for volume in volumes:
            if volume not in self.volumes:
                self.volumes.append(volume)
        self.volume = volumes[0]
        return volumes

    def attach_test_volume(self,volume=None):
        volume = volume or self.volume
        try:
            self.instance.attach_volume(volume, timeout=90)
        except ec2ops.VolumeStateException, vse:
            self.status("This is a temp work around for testing, this is to avoid bug euca-5297"+str(vse),
                        testcolor=TestColor.get_canned_color('failred'))
            time.sleep(10)
            self.debug('Monitoring volume post VolumeStateException...')
            volume.eutest_attached_status = None
            self.tester.monitor_euvolumes_to_status([volume],status='in-use',attached_status='attached',timeout=90)


    def mount_volume_and_create_test_file(self, force_mkfs=True):
        mount_point = self.instance.mount_attached_volume(volume=self.volume,force_mkfs=force_mkfs)
        mount_point = str(mount_point).rstrip('/')+'/'
        test_file_path = mount_point+ 'testfile'
        self.instance.sys('touch ' + test_file_path, code=0)
        self.test_file_path = test_file_path

    def get_mpath_info_for_instance_vol(self, instance, vol):
        if not hasattr(instance, 'node'):
            instance.node = self.get_node_instance_is_running_on(instance)
        info = instance.node.get_instance_multipath_dev_info_for_instance_ebs_volume(instance, vol)
        return info

    def print_mpath_info_for_instance_vol(self, instance, vol, mute=False):
        info = "\n".join(self.get_mpath_info_for_instance_vol(instance, vol))
        line = "\n-----------------------------------------------------------------------------------\n"
        header = "\nMPATH INFO --> INSTANCE:" + str(instance.id) + ", VOLUME:" + str(vol.id) + "\n"
        buf = line + header + info + line
        if not mute:
            self.debug(buf, linebyline=False)
        return (buf)

    def sftp_test_io_script_to_instance(self):
        local_script_path =str(self.args.io_script_path)
        remote_script_path = str(os.path.basename(local_script_path))
        self.instance.ssh.sftp_put(local_script_path,remote_script_path)
        self.remote_script_path = remote_script_path


    def setup_instance_volume_and_script(self):
        self.get_test_instance()
        self.get_test_volumes()
        self.attach_test_volume()
        self.mount_volume_and_create_test_file()
        self.sftp_test_io_script_to_instance()



    def run_remote_script_and_monitor(self,timed_run=0):
        tb = None
        err = None
        exit_value = None
        exit_lines = ""

        original_inter_io_timeout = self.inter_io_timeout
        if timed_run:
            original_inter_io_timeout = self.inter_io_timeout
        path_controller = self.instance.path_controller
        self.path_controller = path_controller
        path_controller.clear_all_eutester_rules()
        path_controller.total_path_iterations=0

        cmd = 'python ' + self.remote_script_path + " -f " + self.test_file_path + ' -t ' + str(timed_run)
        try:
            self.stdscr = curses.initscr()
        except Exception, e:
            self.debug('No term cababilties? Curses could not init:' +str(e))
        try:
            signal.signal(signal.SIGWINCH, self.sigwinch_handler)
            now = time.time()
            self.test_start_time = now
            out = self.instance.ssh.cmd(cmd,
                                  verbose=False,
                                  cb=self.remote_ssh_io_script_monitor_cb,
                                  cbargs=[None,None,None,None,now, now, 'Starting', None ])
            exit_value = out['status']
            exit_lines = out['output']
        except Exception, e:
            tb = self.tester.get_traceback()
            debug_string = str(tb) + '\nError caught by remote_ssh_io_monitor_cb:'+str(e)
            self.debug(debug_string)
            if self.stdscr:
                self.stdscr.addstr(0, 0, debug_string)
                self.stdscr.refresh()
            err = str(e)
            raise Exception(str(tb)+ '\nError:' + str(err))
        finally:
            self.test_end_time = time.time()
            if exit_value is None or exit_value != 0:
                tb = self.tester.get_traceback()
                err = "Remote io script ended with invalid status code:" + str(exit_value) + "\n" + str(exit_lines)
            self.tear_down_curses()
            try:
                path_controller.clear_all_eutester_rules()
            except Exception, pe:
                tb += "\n" + str(pe)
            if tb:
                raise Exception(str(tb)+ '\nError:' + str(err))
            self.debug('Remote Monitor finished successfully, final output:' + "\n".join(exit_lines))


    def tear_down_curses(self):
        if self.stdscr:
            self.stdscr.keypad(0)
            curses.echo()
            curses.nocbreak()
            curses.endwin()
            self.stdscr = None


    #@eutester.Eutester.printinfo
    def remote_ssh_io_script_monitor_cb(self,
                                        buf,
                                        write_value,
                                        write_rate,
                                        read_rate,
                                        last_read,
                                        last_time,
                                        cycle_check_time,
                                        status,
                                        mpath_status):
        ret = SshCbReturn(stop=False, settimer=self.inter_io_timeout)
        return_buf = ""
        now = time.time()
        path_controller = self.instance.path_controller
        remaining_iterations = self.max_path_iterations- path_controller.total_path_iterations
        completed_iterations = "COMPLETED_ITERATIONS:" + str(path_controller.total_path_iterations)
        remaining_iterations_str = "REMAINING_ITERATIONS:" + str(remaining_iterations)
        write_rate = write_rate or 'WRITE_RATE:'
        write_value = write_value or 'WRITE_VALUE'
        read_rate = read_rate or 'READ_RATE'
        last_read = last_read or 'LAST_READ'

        last_time = last_time or now
        waited = int(now - last_time)
        waited_str = "INTER_IO_SECONDS_WAITED: "+ str(waited)
        time_remaining = "TIME_REMAINING:"
        printout = False
        #Pace cycle checks
        if now - cycle_check_time >= 5:
            status = self.current_cycle_method()
            cycle_check_time = now
            try:
                printout = True
                mpath_status = self.print_mpath_info_for_instance_vol(self.instance, self.volume)
            except Exception, e:
                mpath_status = str(e)

        status_str = 'STATUS:' + str(status)

        if waited > self.longest_wait_period:
            self.longest_wait_period = waited
        longest_wait_period_str = "LONGEST_PERIOD_WAITED:" +str(self.longest_wait_period)
        try:
            for line in str(buf).splitlines():
                if re.match('WRITE_VALUE',line):
                    write_value = line
                elif re.match('WRITE_RATE', line):
                    write_rate = line
                elif re.match('READ_RATE', line):
                    read_rate = line
                elif re.match('LAST_READ', line):
                    last_read = line
                elif re.match('TIME_REMAINING', line):
                    time_remaining = line
                elif re.search('err', line, re.IGNORECASE):
                    return_buf += line

            debug_string = "Instance: " + str(self.instance.id) + ", Volume:" + str(self.volume.id )+ "\n" \
                           + "-------------------------------------------------\n" \
                           + write_value.ljust(20) + "\n" \
                           + write_rate.ljust(30) + "\n" \
                           + read_rate.ljust(30) + "\n" \
                           + last_read.ljust(20) + "\n" \
                           + waited_str.ljust(20) + "\n" \
                           + str(longest_wait_period_str).ljust(20) + "\n" \
                           + str(completed_iterations) + "\n" \
                           + str(remaining_iterations_str) + "\n" \
                           + str(time_remaining) + "\n" \
                           + str(status_str) + "\n" \
                           + str(mpath_status) + "\n" \
                           + "ret buf:" + str(return_buf) \
                           + "\n-------------------------------------------------\n"


            if remaining_iterations <= 0 or status == self.stopped_status:
                debug_string += '\nREMOTE MONITOR FINSIHED SUCCESSFULLY\n'
                self.tear_down_curses()
                self.debug(debug_string)
                ret.statuscode = 0
                ret.stop = True
            else:
                #print "\r\x1b[K"+str(debug_string),
                #sys.stdout.flush()
                if self.stdscr:
                    self.stdscr.clear()
                    self.stdscr.addstr(0, 0, debug_string)
                    self.stdscr.refresh()
                elif printout:
                    self.debug(debug_string +
                               str('--------------------------------------------------------------------------------'))
        except Exception, e:
            tb = self.tester.get_traceback()
            debug_string = str(tb) + '\nError caught by remote_ssh_io_monitor_cb:'+str(e)
            self.tear_down_curses()
            self.errormsg(debug_string)
            ret.stop = True
            ret.statuscode=69
            pass
        finally:
            ret.nextargs = [ write_value, write_rate,read_rate, last_read, time.time(), cycle_check_time, status, mpath_status]
            ret.buf = return_buf
            return ret



    def sigwinch_handler(self, signal, frame ):
        if self.stdscr:
            curses.endwin()
            self.stdscr = curses.initscr()

    def check_mpath_iterations(self):
        if self.path_controller and self.path_controller.remaining_iterations:
            remaining = self.path_controller.remaining_iterations
            raise Exception('Path Controller did not complete its iterations. Remaining:'+str(remaining))

    def test1_check_volume_io_on_guest_while_blocking_clearing_all_paths_once(self,clean_on_exit=True):
        test_list = []
        errmsg = ''
        self.pre_test_check_should_run_nc_multipath_tests_on_this_system()
        try:
            #Setup and connect to instance, create + attach vol, format vol, scp io script, create test dir/file.
            self.setup_instance_volume_and_script()

            self.max_path_iterations=1
            #Run the remote io script on the test instance, monitor all script output via local call back method
            self.run_remote_script_and_monitor()
        except Exception, e:
            tb = self.tester.get_traceback()
            errmsg = "ERROR:\n" + str(tb) + "\n" + str(e)
            self.debug(errmsg)
        finally:
            if clean_on_exit:
                try:
                    for pc in self.path_controllers:
                        pc.clear_all_eutester_rules()
                    self.instance.terminate_and_verify()
                except Exception, ie:
                    errmsg += '\n' + str(ie)
            if errmsg:
                raise Exception(errmsg)



    def test2_attach_volume_while_a_single_path_is_down(self, wait_after_block=60, mb=1, clean_on_exit=True):
        single_path = None
        length = mb * 1048576
        errmsg = ''
        path_controller = None
        self.pre_test_check_should_run_nc_multipath_tests_on_this_system()
        try:
            self.status('Get test instance to run test...')
            self.get_test_instance()
            path_controller =  self.instance.path_controller
            if len(path_controller.sp_ip_list) > 1:
                single_path = path_controller.sp_ip_list[1]
            else:
                raise Exception('Not enough paths to shut one down for test')
            self.status('Getting volume for use in test...')
            self.get_test_volumes()
            self.status('Checking paths to make sure none are currently blocked...')
            if path_controller.get_blocked_paths():
                self.debug('Clearing blocked paths and sleeping for recovery period:' +
                           str(self.path_recovery_interval) + ' seconds')
                path_controller.clear_all_eutester_rules(timeout=120)
                time.sleep(self.path_recovery_interval)
            self.status('Blocking single path: "' + str(single_path) + '" before attaching volume...')
            path_controller.block_path(single_path)
            self.status('Path is blocked waiting ' + str(wait_after_block) + 'seconds before attach...')
            time.sleep(wait_after_block)
            self.status('Path is blocked, attaching volume...')
            self.attach_test_volume()
            self.print_mpath_info_for_instance_vol(self.instance, self.volume)

            self.status('Mounting volume and creating guest side test env...')

            #self.mount_volume_and_create_test_file()
            #self.sftp_test_io_script_to_instance()
            self.status('Attempting to run some basic io on guest volume...')
            self.instance.vol_write_random_data_get_md5(self.volume, length=length, overwrite=True)
            self.status('Remote io monitor script done, success. Attempting to detach volume now...')
            self.instance.detach_euvolume(self.volume)
        except Exception, e:
            tb = self.tester.get_traceback()
            errmsg = "\n" + str(tb) + "\nERROR: " + str(e)
            self.errormsgg(errmsg)
        finally:
            if clean_on_exit:
                self.status('Tearing down instance after test...')
                try:
                    if path_controller:
                        path_controller.clear_all_eutester_rules()
                    else:
                        for pc in self.path_controllers:
                            pc.clear_all_eutester_rules()
                    self.instance.terminate_and_verify()
                except Exception, ie:
                    errmsg += '\n' + str(ie)
            if errmsg:
                raise Exception(errmsg)

    def test3_attach_volume_while_a_single_path_is_in_process_of_failing(self, clean_on_exit=True):
        single_path = None
        errmsg = ''
        path_controller = None
        self.pre_test_check_should_run_nc_multipath_tests_on_this_system()
        try:
            self.status('Get test instance to run test...')
            self.get_test_instance()
            path_controller =  self.instance.path_controller
            if len(path_controller.sp_ip_list) > 1:
                single_path = path_controller.sp_ip_list[1]
            else:
                raise Exception('Not enough paths to shut one down for test')
            self.status('Getting volume for use in test...')
            self.get_test_volumes()
            self.status('Checking paths to make sure none are currently blocked...')
            if path_controller.get_blocked_paths():
                self.debug('Clearing blocked paths and sleeping for recovery period:' +
                           str(self.path_recovery_interval) + ' seconds')
                path_controller.clear_all_eutester_rules(timeout=120)
                time.sleep(self.path_recovery_interval)
            self.status('Blocking single path: "' + str(single_path) + '" before attaching volume...')
            path_controller.block_path(single_path)
            self.status('Path is blocked, immediately attaching volume while path is in process of failing...')
            self.attach_test_volume()
            self.print_mpath_info_for_instance_vol(self.instance, self.volume)
            self.status('Mounting volume and creating guest side test env...')
            self.mount_volume_and_create_test_file()
            self.sftp_test_io_script_to_instance()
            self.status('Attempting to run some basic io on guest volume...')
            self.instance.vol_write_random_data_get_md5(self.volume, length=1048576, overwrite=True)
            self.status('Remote io done, detaching...')
            self.instance.detach_euvolume(self.volume)
        except Exception, e:
            tb = self.tester.get_traceback()
            errmsg = "\n" + str(tb) + "\nERROR: " + str(e)
            self.errormsg(errmsg)
        finally:
            if clean_on_exit:
                self.status('Tearing down instance after test...')
                try:
                    if path_controller:
                        path_controller.clear_all_eutester_rules()
                    else:
                        for pc in self.path_controllers:
                            pc.clear_all_eutester_rules()
                    self.instance.terminate_and_verify()
                except Exception, ie:
                    errmsg += '\n' + str(ie)
            if errmsg:
                raise Exception(errmsg)



    def test4_detach_volume_with_single_path_down_after_attached(self, wait_after_block=60, clean_on_exit=True):
        single_path = None
        errmsg = ''
        path_controller = None
        self.pre_test_check_should_run_nc_multipath_tests_on_this_system()
        try:
            self.status('Get test instance to run test...')
            self.get_test_instance()
            path_controller =  self.instance.path_controller
            if len(path_controller.sp_ip_list) > 1:
                single_path = path_controller.sp_ip_list[1]
            else:
                raise Exception('Not enough paths to shut one down for test')
            self.status('Getting volume for use in test...')
            self.get_test_volumes()
            self.status('Checking paths to make sure none are currently blocked...')
            if path_controller.get_blocked_paths():
                self.debug('Clearing blocked paths and sleeping for recovery period:' +
                           str(self.path_recovery_interval) + ' seconds')
                path_controller.clear_all_eutester_rules(timeout=120)
                time.sleep(self.path_recovery_interval)

            self.status('Attaching test volume while all paths are up...')
            self.attach_test_volume()
            self.print_mpath_info_for_instance_vol(self.instance, self.volume)
            self.status('Mounting volume and creating guest side test env...')
            self.mount_volume_and_create_test_file()
            self.sftp_test_io_script_to_instance()
            self.status('Attempting to run some basic io on guest volume...')
            self.instance.vol_write_random_data_get_md5(self.volume, length=1048576, overwrite=True)
            self.status('Remote write/read done')
            self.status('Blocking single path: "' + str(single_path) + '" before detaching volume:'
                        + str(self.volume.id) + '...')
            path_controller.block_path(single_path)
            self.status('Waiting ' + str(wait_after_block) + ' seconds after blocking for path to go down...')
            time.sleep(wait_after_block)
            self.status('Detaching volume while path is down...')
            self.instance.detach_euvolume(self.volume)

        except Exception, e:
            tb = self.tester.get_traceback()
            errmsg = "\n" + str(tb) + "\nERROR: " + str(e)
            self.errormsg(errmsg)
        finally:
            if clean_on_exit:
                self.status('Tearing down instance after test...')
                try:
                    if path_controller:
                        path_controller.clear_all_eutester_rules()
                    else:
                        for pc in self.path_controllers:
                            pc.clear_all_eutester_rules()
                    self.instance.terminate_and_verify()
                except Exception, ie:
                    errmsg += '\n' + str(ie)
            if errmsg:
                raise Exception(errmsg)


    def test5_attach_volume_while_a_single_path_is_down_run_io_monitor(self, wait_after_block=60, mb=1, clean_on_exit=True):
        single_path = None
        length = mb * 1048576
        errmsg = ''
        path_controller = None
        self.pre_test_check_should_run_nc_multipath_tests_on_this_system()
        try:
            self.status('Get test instance to run test...')
            self.get_test_instance()
            path_controller =  self.instance.path_controller
            if len(path_controller.sp_ip_list) > 1:
                single_path = path_controller.sp_ip_list[1]
            else:
                raise Exception('Not enough paths to shut one down for test')
            self.status('Getting volume for use in test...')
            self.get_test_volumes()
            self.status('Checking paths to make sure none are currently blocked...')
            if path_controller.get_blocked_paths():
                self.debug('Clearing blocked paths and sleeping for recovery period:' +
                           str(self.path_recovery_interval) + ' seconds')
                path_controller.clear_all_eutester_rules(timeout=120)
                time.sleep(self.path_recovery_interval)
            self.status('Blocking single path: "' + str(single_path) + '" before attaching volume...')
            path_controller.block_path(single_path)
            self.status('Path is blocked waiting ' + str(wait_after_block) + 'seconds before attach...')
            time.sleep(wait_after_block)
            self.status('Path is blocked, attaching volume...')
            self.attach_test_volume()
            self.print_mpath_info_for_instance_vol(self.instance, self.volume)
            self.status('Mounting volume and creating guest side test env...')

            self.mount_volume_and_create_test_file()
            self.sftp_test_io_script_to_instance()
            self.status('Attempting to run some basic io on guest volume...')
            self.run_remote_script_and_monitor()
            #self.instance.vol_write_random_data_get_md5(self.volume, length=length, overwrite=True)
            self.status('Remote io monitor script done, success. Attempting to detach volume now...')
            self.instance.detach_euvolume(self.volume)
        except Exception, e:
            tb = self.tester.get_traceback()
            errmsg = "\n" + str(tb) + "\nERROR: " + str(e)
            self.errormsg(errmsg)
        finally:
            if clean_on_exit:
                self.status('Tearing down instance after test...')
                try:
                    if path_controller:
                        path_controller.clear_all_eutester_rules()
                    else:
                        for pc in self.path_controllers:
                            pc.clear_all_eutester_rules()
                    self.instance.terminate_and_verify()
                except Exception, ie:
                    errmsg += '\n' + str(ie)
            if errmsg:
                raise Exception(errmsg)



    def test6_attach_vol_while_up_and_attach_vol_while_down_detach_both_cases(self,
                                                                              wait_after_block=60,
                                                                              vols_before_block=3,
                                                                              vols_after_block=2,
                                                                              clean_on_exit=True):
        '''
        Attaches 'vols_before_block' number of volumes while paths are not blocked. Then blocks a single path
        and attaches 'vols_after_block' number of volumes while a single path has been blocked.
        Detaches a volume attached before blocking as well as a volume attached after blocking from the instance.
        Clears all paths and detaches all remaining volumes from the instance.
        terminates instance.
        '''

        single_path = None
        before_block = []
        after_block = []
        errmsg = ''
        path_controller = None
        self.pre_test_check_should_run_nc_multipath_tests_on_this_system()
        try:
            self.status('Get test instance to run test...')
            self.get_test_instance()
            path_controller =  self.instance.path_controller
            if len(path_controller.sp_ip_list) > 1:
                single_path = path_controller.sp_ip_list[1]
            else:
                raise Exception('Not enough paths to shut one down for test')
            self.status('Getting volume for use in test...')
            volumes = self.get_test_volumes(count=int(vols_after_block + vols_before_block))
            self.tester.print_euvolume_list(volumes)
            after_block = copy.copy(volumes)
            self.status('Checking paths to make sure none are currently blocked...')
            if path_controller.get_blocked_paths():
                self.debug('Clearing blocked paths and sleeping for recovery period:' +
                           str(self.path_recovery_interval) + ' seconds')
                path_controller.clear_all_eutester_rules(timeout=120)
                time.sleep(self.path_recovery_interval)

            for x in xrange(0, vols_before_block):
                volume = after_block.pop()
                before_block.append(volume)
                self.status('Attaching test volume ' + str(x+1) + ' of ' +str(vols_before_block) + ' while all paths are up...')
                self.attach_test_volume(volume)
                self.print_mpath_info_for_instance_vol(self.instance, volume)
                self.status('Attempting to run some basic io on guest volume...')
                self.instance.vol_write_random_data_get_md5(volume, length=1048576, overwrite=True)
                self.status('Remote write/read done for volume:' +str(volume.id))
            self.status('Attached all ' + str(vols_before_block) + ' volumes before blocking a path')
            self.tester.print_euvolume_list(volumes)
            if after_block:
                self.status('Blocking single path: "' + str(single_path) + "...")
                path_controller.block_path(single_path)
                self.status('Waiting ' + str(wait_after_block) + ' seconds after blocking for path to go down...')
                time.sleep(wait_after_block)
                #Iterate through remaining volumes in list, rename
                for volume in after_block:
                    self.status('Attaching test volume ' + str(after_block.index(volume)) + ' of ' +str(len(after_block)) +
                                'while path:' + str(single_path) + " is down")
                    self.attach_test_volume(volume)
                    self.print_mpath_info_for_instance_vol(self.instance, volume)
                    self.status('Attempting to run some basic io on guest volume...')
                    self.instance.vol_write_random_data_get_md5(volume, length=1048576, overwrite=True)
                    self.status('Remote write/read done for volume:'  + str(volume.id) )
            self.tester.print_euvolume_list(volumes)
            self.status('Detaching a volume while path is down...')
            detach_vols = []
            #Grab a volume from each list to detach
            if before_block:
                detach_vols.append(before_block.pop())
            if after_block:
                detach_vols.append(after_block.pop())
            self.status('Detaching volume(s) from before and after blocking a path(s)')
            for volume in detach_vols:
                self.instance.detach_euvolume(volume)
            if before_block or after_block:
                detach_vols = []
                if before_block:
                    detach_vols.append(before_block.pop())
                if after_block:
                    detach_vols.append(after_block.pop())
                self.status('Clearing blocked paths to detach remaining volumes')
                self.status('Checking paths to make sure none are currently blocked...')
                if path_controller.get_blocked_paths():
                    self.debug('Clearing blocked paths and sleeping for recovery period:' +
                               str(self.path_recovery_interval) + ' seconds')
                    path_controller.clear_all_eutester_rules(timeout=120)
                    time.sleep(self.path_recovery_interval)
                    for volume in detach_vols:
                        self.instance.detach_euvolume(volume)
            before_block.extend(after_block)
            self.tester.print_euvolume_list(volumes)
            self.status('terminating instance with ' + str(len(before_block)) + ' volumes attached')

        except Exception, e:
            tb = self.tester.get_traceback()
            errmsg = "\n" + str(tb) + "\nERROR: " + str(e)
            self.errormsg(errmsg)
        finally:
            if clean_on_exit:
                self.status('Tearing down instance after test...')
                try:
                    if path_controller:
                        path_controller.clear_all_eutester_rules()
                    else:
                        for pc in self.path_controllers:
                            pc.clear_all_eutester_rules()
                    self.instance.terminate_and_verify()
                except Exception, ie:
                    errmsg += '\n' + str(ie)
            if errmsg:
                raise Exception(errmsg)

    def test7_test_storage_controller_sanhost_fallback(self,zone=None, wait_for_host=120):
        zone = zone or self.zone
        errmsg=""
        self.pre_test_check_should_run_sc_multihosts_tests_on_this_system(zone)
        self.status('Attempting to get enabled sc and sanhosts from zone:' +str(zone))
        sc = self.get_enabled_storage_controller_for_zone(zone=zone)
        sanhosts = self.get_sc_sanhosts_by_sc(sc)
        if not len(sanhosts) > 1:
            raise SkipTestException('Skipping test - Did not find > 1 host in sanhost property:' + str(sanhosts))
        pc = Path_Controller(node=sc,sp_ip_list=sanhosts)
        if pc.get_blocked_paths():
            pc.clear_all_eutester_rules()
            time.sleep(60)
        self.status('Attempting to iterate through list of sanhosts, block host, create vol, then clear host...')
        hostcount=0
        try:
            for host in sanhosts:
                hostcount += 1
                self.status('Attempting to block host: ' + str(host) + ' count(' + str(hostcount) + '/' + str(len(sanhosts)) + ')')
                pc.block_path(host)
                volume = self.tester.create_volume(zone=zone, size=1)
                self.tester.delete_volume(volume=volume, timeout=300)
                self.status('Done with host:' + str(host) + ', clearing rules and sleeping for ' + str(wait_for_host) +' seconds')
                pc.clear_all_eutester_rules()
                time.sleep(wait_for_host)
            self.status('Success - Created a volume while blocking network i/o to each sanhost')
            self.debug('Make sure all paths are unblocked on SC...')
        except Exception, e:
            tb = self.tester.get_traceback()
            errmsg += '\n' + str(tb) + '\n Caught error:' + str(e)
            self.debug(errmsg, linebyline=False)
        finally:
            try:
                pc.clear_all_eutester_rules()
            except Exception, ce:
                errmsg += '\n' + str(ce)
            if errmsg:
                raise Exception(errmsg)


    def create_local_test_volume_checksum(self, instance):
        gb =1073741824
        mb = 1048576
        self.status('Getting a unique md5 sum for volume...')
        vol = self.get_test_volumes()[0]
        instance.attach_volume(volume=vol)
        dd_dict = instance.random_fill_volume(vol,length=mb)
        wrote1 = int(dd_dict['dd_bytes'])
        dd_dict2 = instance.dd_monitor(ddif='/dev/zero', ddof=vol.guestdev, ddbytes= (gb - wrote1), ddseek=dd_dict['dd_bytes'] )
        wrote2 = int(dd_dict2['dd_bytes'])
        instance.get_dev_md5(vol.guestdev, length=(wrote1+wrote2), timeout=90)
        self.testvol_w_checksum = vol
        return vol

    def test8_test_storage_controller_path_fail_then_create_snapshot(self,):
        errmsg = ""
        self.pre_test_check_should_run_sc_multipath_tests_on_this_system()
        self.status('Get test instance to run test...')
        instance = self.get_test_instance()
        zone = str(instance.placement)
        self.status('Attempting to get enabled sc and scpaths from zone:' +str(zone))
        sc = self.get_enabled_storage_controller_for_zone(zone=zone)
        paths = self.get_sc_paths_by_sc(sc)
        pc = Path_Controller(node=sc,sp_ip_list=paths)
        self.status('Clearing all eutester rules on: ' + str(pc.host))
        if pc.get_blocked_paths():
            pc.clear_all_eutester_rules()
            time.sleep(60)
        vol = self.testvol_w_checksum or self.create_local_test_volume_checksum(instance=instance)
        paths_to_block = copy.copy(pc.sp_ip_list)
        dont_block = paths_to_block.pop()
        paths_to_block_string =",".join(paths_to_block)
        self.status('Unblocked:' + str(dont_block) + ', Blocking paths:' + str(paths_to_block_string) + ', on SC:' + str(pc.host))
        try:
            for path in paths_to_block:
                pc.block_path(path)
            self.status('Creating Snapshot with path ' + str(paths_to_block_string) + ', blocked...')
            test_snap = self.tester.create_snapshots(volume=vol, wait_on_progress=40, monitor_to_completed=True)[0]
            self.status('Created snapshot, now verifying snapshot integrity from volume md5 check...')
            self.status('Creating volume from test snapshot...')
            vol_from_snap = self.tester.create_volume(zone=self.zone, snapshot=test_snap)
            self.status('Attaching volume created from test snapshot to verify md5sum against original...')
            instance.attach_volume(volume=vol_from_snap)
            instance.md5_attached_euvolume(vol_from_snap, length=vol.md5len)
            if vol.md5 != vol_from_snap.md5:
                raise Exception('Vol created from snapshot md5(' + str(vol_from_snap.md5)+'/' + str(vol_from_snap.md5len) +
                                ') != original volume(' + str(vol.md5) + '/' + str(vol.md5len) + ')')
            else:
                self.debug('SUCCESS: Vol created from snapshot md5(' + str(vol_from_snap.md5)+'/' + str(vol_from_snap.md5len) +
                           ') == original volume(' + str(vol.md5) + '/' + str(vol.md5len) + ')')
        except Exception, e:
            tb = self.tester.get_traceback()
            errmsg += '\n' + str(tb) + '\n Caught error:' + str(e)
            self.debug(errmsg,linebyline=False)
        finally:
            try:
                pc.clear_all_eutester_rules()
            except Exception, ce:
                errmsg += '\n' + str(ce)
            if errmsg:
                raise Exception(errmsg)


    def test9_test_storage_controller_path_fail_while_create_snapshot_in_progress(self):
        errmsg=""
        self.pre_test_check_should_run_sc_multipath_tests_on_this_system()
        self.status('Get test instance to run test...')
        instance = self.get_test_instance()
        zone = str(instance.placement)
        self.status('Attempting to get enabled sc and scpaths from zone:' +str(zone))
        sc = self.get_enabled_storage_controller_for_zone(zone=zone)
        paths = self.get_sc_paths_by_sc(sc)
        if not len(paths) > 1:
            raise SkipTestException('Skipping test - Did not find > 1 path in scpaths property:' + str(paths))
        pc = Path_Controller(node=sc,sp_ip_list=paths)
        self.status('Clearing all eutester rules on: ' + str(pc.host))
        if pc.get_blocked_paths():
            pc.clear_all_eutester_rules()
            time.sleep(60)
        self.status('Getting volume with unique checksum...')
        vol = self.testvol_w_checksum or self.create_local_test_volume_checksum(instance=instance)
        path_to_block = pc.sp_ip_list[0]
        self.status('Creating snapshot, then monitoring to at least 1% progress before blocking path...')
        snap = self.tester.create_snapshots(volume=vol, wait_on_progress=40, monitor_to_completed=False)[0]
        self.tester.monitor_eusnaps_to_completed(snaps=[snap],monitor_to_progress=1)
        snap.update()
        self.status('Snap progress'+ str(snap.progress) +', Blocking path:' + str(path_to_block) + ', on SC:' + str(pc.host))
        try:
            pc.block_path(path_to_block)
            self.status('Monitoring snapshot to completed state...')
            self.tester.monitor_eusnaps_to_completed(snaps=[snap])
            self.status('Created snapshot, now verifying snapshot integrity from volume md5 check...')
            self.status('Creating volume from test snapshot...')
            vol_from_snap = self.tester.create_volume(zone=self.zone, snapshot=snap)
            self.status('Attaching volume created from test snapshot to verify md5sum against original...')
            instance.attach_volume(volume=vol_from_snap)
            instance.md5_attached_euvolume(vol_from_snap, length=vol.md5len)
            if vol.md5 != vol_from_snap.md5:
                raise Exception('Vol created from snapshot md5(' + str(vol_from_snap.md5)+'/' + str(vol_from_snap.md5len) +
                                ') != original volume (' + str(vol.md5) + '/' + str(vol.md5len) + ')')
            else:
                self.debug('SUCCESS: Vol created from snapshot md5(' + str(vol_from_snap.md5)+'/' + str(vol_from_snap.md5len) +
                           ') == original volume(' + str(vol.md5) + '/' + str(vol.md5len) + ')')
        except Exception, e:
            tb = self.tester.get_traceback()
            errmsg += '\n' + str(tb) + '\n Caught error:' + str(e)
            self.debug(errmsg, linebyline=False)
        finally:
            try:
                pc.clear_all_eutester_rules()
            except Exception, ce:
                errmsg += '\n' + str(ce)
            if errmsg:
                raise Exception(errmsg)


    def sc_test_suite(self):
        '''
        SC test suite includes testing dev mapper multipath during snapshot creation, as well as sanhost fallback tests
        '''
        self.cycle_paths = True
        test_list = []
        test_list.append(self.create_testunit_from_method(self.test7_test_storage_controller_sanhost_fallback))
        test_list.append(self.create_testunit_from_method(self.test8_test_storage_controller_path_fail_then_create_snapshot))
        test_list.append(self.create_testunit_from_method(self.test9_test_storage_controller_path_fail_while_create_snapshot_in_progress))
        return test_list

    def test_suite(self):
        '''
        Full test suite includes both NC and SC tests
        '''
        test_list = []
        test_list.extend(self.nc_test_suite())
        test_list.extend(self.sc_test_suite())
        return test_list

    def nc_test_suite(self):
        '''
        NC test suite includes testing volume attach, detach, and i/o operations during different stages of path fail over
        and recovery.
        '''
        self.cycle_paths = True
        test_list = []
        #test_list.append(self.create_testunit_from_method(self.pre_test_check_should_run_multipath_tests_on_this_system, eof=True))
        test_list.append(self.create_testunit_from_method(self.test1_check_volume_io_on_guest_while_blocking_clearing_all_paths_once, eof=True))
        test_list.append(self.create_testunit_from_method(self.test2_attach_volume_while_a_single_path_is_down))
        test_list.append(self.create_testunit_from_method(self.test3_attach_volume_while_a_single_path_is_in_process_of_failing))
        test_list.append(self.create_testunit_from_method(self.test4_detach_volume_with_single_path_down_after_attached))
        test_list.append(self.create_testunit_from_method(self.test5_attach_volume_while_a_single_path_is_down_run_io_monitor))
        test_list.append(self.create_testunit_from_method(self.test6_attach_vol_while_up_and_attach_vol_while_down_detach_both_cases))
        return test_list

if __name__ == "__main__":
    testcase = Mpath_Suite()

    ### Use the list of tests passed from config/command line to determine what subset of tests to run
    ### or use a predefined list
    if testcase.args.run_suite:
        unit_list = testcase.test_suite()
    else:
        test_names = testcase.args.tests or ['test1_check_volume_io_on_guest_while_blocking_clearing_all_paths_once']
        ### Convert test suite methods to EutesterUnitTest objects
        unit_list = [ ]
        for test in test_names:
            unit_list.append(testcase.create_testunit_by_name(test))

    ### Run the EutesterUnitTest objects
    result = testcase.run_test_case_list(unit_list,eof=False,clean_on_exit=True)
    sys.exit(result)












import urlparse
import httplib
import base64
from hashlib import sha1
import hmac
from email.utils import formatdate

def lowercase_key(d):
    return dict((k.lower(), v) for k, v in d.iteritems())

def amz_headers(h):
    x_amz_keys = sorted(k for k in h if k.startswith('x-amz-'))
    for key in x_amz_keys:
        values = h[key]
        if isinstance(values, basestring):
            values = [values]

        value = ','.join(s.strip() for s in values)
        yield '%s:%s' % (key, value)

class Auth(object):
    def __init__(self, access_key, secret_key):
        self.access_key = access_key
        self.secret_key = secret_key

    def canonicalize(self, verb, resource, headers, x_amz_headerlist=None):
        if not resource.startswith('/'):
            resource = '/' + resource
        headers = lowercase_key(headers)
        signature_elements = [
            verb,
            headers.get("content-md5", ""),
            headers.get("content-type", ""),
            headers.get("date", ""),
        ]
        if x_amz_headerlist is None:
            x_amz_headerlist = amz_headers(headers)

        signature_elements.extend(x_amz_headerlist)
        signature_elements.append(resource)
        return '\n'.join(signature_elements)

    def sign(self, signature_str):
        h = hmac.new(self.secret_key, signature_str, sha1)
        d = h.digest()
        return base64.encodestring(d).strip()

    def header(self, verb, resource, headers, x_amz_headerlist):
        signature_str = self.canonicalize(verb, resource, headers, x_amz_headerlist)
        ####print "SIGNATURE", signature_str
        signature = self.sign(signature_str)
        return 'AWS %s:%s' % (self.access_key, signature)

class S3Connection(object):
    CONNECTOR = {
        'http': httplib.HTTPConnection,
        'https': httplib.HTTPSConnection,
    }

    def __init__(self, url, auth):
        self.auth = auth

        parts = urlparse.urlsplit(url)
        connector = self.CONNECTOR[parts.scheme]
        self.netloc = parts.netloc

        host, _, _ = parts.netloc.partition(':')
        port = parts.port
        self.basepath = parts.path.rstrip('/')

        self.conn = connector(host, port, timeout=15)
        
    def putheaders(self, headers):
        for header, values in headers.iteritems():
            if isinstance(values, basestring):
                values = [values]
            for v in values:
                self.conn.putheader(header, v)

    @property
    def is_aws(self):
        return 'amazonaws.com' in self.netloc

    # Guess the proper Host-header based on host
    def bucket_host(self, bucket):
        if self.is_aws:
            return bucket + '.' + self.netloc
        return bucket + '.walrus'
    
    def fix_resource(self, resource):
        if self.is_aws:
            return resource
        return '/services/Walrus' + resource

    def request(self, method, url, resource, data, headers, vhost=None, x_amz_headerlist=None):
        full_path = self.basepath + url

        headers.setdefault("Content-Type", "text/plain")
        headers.setdefault("Date", formatdate(usegmt=True))

        headers['Authorization'] = self.auth.header(method, resource, headers, x_amz_headerlist)
        headers['Content-Length'] = str(len(data))

        headers['Host'] = self.bucket_host(vhost) if vhost else self.netloc 

        ####self.conn.set_debuglevel(1)
        self.conn.connect()
        self.conn.putrequest(method, full_path, skip_host=True)
        self.putheaders(headers)
        self.conn.endheaders()
        self.conn.send(data)

        resp = self.conn.getresponse()
        status, data = resp.status, resp.read()
        self.conn.close()
        
        return status, data




__author__ = 'Swathi Gangisetty'

import unittest
from eucaops import Eucaops
import argparse

class EUCA2782(unittest.TestCase):

    def setUp(self):
        self.tester = eucaops

    def test_EUCA2782(self):
        ### Create a bucket in Walrus
        self.bucket_obj = self.tester.create_bucket('bucket_euca2782')

        ### Make a HEAD request against the bucket that was previously created and a key that does not exist
        response = self.tester.s3.make_request(method='HEAD', bucket=self.bucket_obj, key='key')

        ### Assert that the HTTP response has no message body and status is 404
        self.assertEqual(response.status, 404, 'Expected response status code to be 404, actual status code is ' + str(response.status))
        self.assertEqual(response.length, 0, 'Expected response length to be 0, actual length is ' + str(response.length))

        ### Delete the bucket
        self.tester.delete_bucket(bucket=self.bucket_obj)

    def test_EUCA2241(self):
        ### Create a bucket in Walrus
        self.bucket_obj = self.tester.s3.create_bucket('bucket_euca2241')

        ### GET a key that does not exist
        response = self.bucket_obj.get_key('key')

        ### Assert that the response is null
        self.assertEqual(response, None)

        ### Delete the bucket
        response = self.tester.s3.delete_bucket(bucket=self.bucket_obj)

        ### Assert that the response is null
        self.assertEqual(response, None)

        ### Make a HEAD request against the bucket that was deleted
        response = self.tester.s3.make_request(method='GET', bucket=self.bucket_obj)
        self.assertEqual(response.status, 404, 'Expected response status code to be 404, actual status code is ' + str(response.status))

    def tearDown(self):
        self.tester=None

if __name__ == "__main__":
    ### Parse command line arguments and get the path to credentials
    parser = argparse.ArgumentParser(description='Unit test to verify fix for EUCA-2782 and EUCA-2241')
    parser.add_argument('--credpath', dest='credpath', required=True, help='Path to folder containing credentials')
    args = parser.parse_args()

    try:
        ### Setup the cloud connection
        eucaops = Eucaops(credpath=args.credpath)
    except Exception, e:
        exit('Failed to establish connection to cloud due to: ' + str(e))

    ### Run the unit test
    suite = unittest.TestLoader().loadTestsFromTestCase(EUCA2782)
    unittest.TextTestRunner(verbosity=2).run(suite)

    eucaops=None


__author__ = 'Swathi Gangisetty'

import unittest
from eucaops import Eucaops
import argparse

key_with_star_name = 'special characters%unravelled_bigtime/abc*'
key_with_dollar_name = 'special characters%unravelled_bigtime/abc$'
key_with_exclamation_name = 'special characters%unravelled_bigtime/abc!'
key_with_percent_name = 'special characters%unravelled_bigtime/abc%'
key_with_underscore_name = 'special characters%unravelled_bigtime/abc_'
key_with_backtick_name = 'special characters%unravelled_bigtime/abc~'
key_with_atrate_name = 'special characters%unravelled_bigtime/abc@'
key_with_pound_name = 'special characters%unravelled_bigtime/abc#'
key_with_caret_name = 'special characters%unravelled_bigtime/abc^'
key_with_ampersand_name = 'special characters%unravelled_bigtime/abc&'
key_with_leftbrace_name = 'special characters%unravelled_bigtime/abc('
key_with_rightbrace_name = 'special characters%unravelled_bigtime/abc)'

class EUCA3453(unittest.TestCase):

    def setUp(self):
        ### Create a bucket in Walrus
        self.tester = eucaops
        self.bucket_obj = self.tester.create_bucket('bucket_euca3453')

    def test_1(self):
        ### Create keys and set content
        key_with_star = self.bucket_obj.new_key(key_name=key_with_star_name)
        key_with_star.set_contents_from_string('key with star')

        key_with_dollar = self.bucket_obj.new_key(key_name=key_with_dollar_name)
        key_with_dollar.set_contents_from_string('key with dollar')

        ### Get keys and check content
        key_with_star = self.bucket_obj.get_key(key_name=key_with_star_name)
        self.assertEqual(key_with_star.read(), 'key with star')

        key_with_dollar = self.bucket_obj.get_key(key_name=key_with_dollar_name)
        self.assertEqual(key_with_dollar.read(), 'key with dollar')

        ### Delete keys
        self.bucket_obj.delete_key(key_name=key_with_star_name)
        self.bucket_obj.delete_key(key_name=key_with_dollar_name)

    def test_2(self):
        ### Create keys and set content
        key_with_exclamation = self.bucket_obj.new_key(key_name=key_with_exclamation_name)
        key_with_exclamation.set_contents_from_string('key with exclamation')

        key_with_percent = self.bucket_obj.new_key(key_name=key_with_percent_name)
        key_with_percent.set_contents_from_string('key with percent')

        ### Get keys and check content
        key_with_exclamation = self.bucket_obj.get_key(key_name=key_with_exclamation_name)
        self.assertEqual(key_with_exclamation.read(), 'key with exclamation')

        key_with_percent = self.bucket_obj.get_key(key_name=key_with_percent_name)
        self.assertEqual(key_with_percent.read(), 'key with percent')

        ### Delete keys
        self.bucket_obj.delete_key(key_name=key_with_exclamation_name)
        self.bucket_obj.delete_key(key_name=key_with_percent_name)

    def test_3(self):
        ### Create keys and set content
        key_with_underscore = self.bucket_obj.new_key(key_name=key_with_underscore_name)
        key_with_underscore.set_contents_from_string('key with underscore')

        key_with_backtick = self.bucket_obj.new_key(key_name=key_with_backtick_name)
        key_with_backtick.set_contents_from_string('key with backtick')

        ### Get keys and check content
        key_with_underscore = self.bucket_obj.get_key(key_name=key_with_underscore_name)
        self.assertEqual(key_with_underscore.read(), 'key with underscore')

        key_with_backtick = self.bucket_obj.get_key(key_name=key_with_backtick_name)
        self.assertEqual(key_with_backtick.read(), 'key with backtick')

        ### Delete keys
        self.bucket_obj.delete_key(key_name=key_with_underscore_name)
        self.bucket_obj.delete_key(key_name=key_with_backtick_name)

    def test_4(self):
        ### Create keys and set content
        key_with_atrate = self.bucket_obj.new_key(key_name=key_with_atrate_name)
        key_with_atrate.set_contents_from_string('key with at rate')

        key_with_pound = self.bucket_obj.new_key(key_name=key_with_pound_name)
        key_with_pound.set_contents_from_string('key with pound')

        ### Get keys and check content
        key_with_atrate = self.bucket_obj.get_key(key_name=key_with_atrate_name)
        self.assertEqual(key_with_atrate.read(), 'key with at rate')

        key_with_pound = self.bucket_obj.get_key(key_name=key_with_pound_name)
        self.assertEqual(key_with_pound.read(), 'key with pound')

        ### Delete keys
        self.bucket_obj.delete_key(key_name=key_with_atrate_name)
        self.bucket_obj.delete_key(key_name=key_with_pound_name)

    def test_5(self):
        ### Create keys and set content
        key_with_caret = self.bucket_obj.new_key(key_name=key_with_caret_name)
        key_with_caret.set_contents_from_string('key with caret')

        key_with_ampersand = self.bucket_obj.new_key(key_name=key_with_ampersand_name)
        key_with_ampersand.set_contents_from_string('key with ampersand')

        ### Get keys and check content
        key_with_caret = self.bucket_obj.get_key(key_name=key_with_caret_name)
        self.assertEqual(key_with_caret.read(), 'key with caret')

        key_with_ampersand = self.bucket_obj.get_key(key_name=key_with_ampersand_name)
        self.assertEqual(key_with_ampersand.read(), 'key with ampersand')

        ### Delete keys
        self.bucket_obj.delete_key(key_name=key_with_caret_name)
        self.bucket_obj.delete_key(key_name=key_with_ampersand_name)

    def test_6(self):
        ### Create keys and set content
        key_with_leftbrace = self.bucket_obj.new_key(key_name=key_with_leftbrace_name)
        key_with_leftbrace.set_contents_from_string('key with left brace')

        key_with_rightbrace = self.bucket_obj.new_key(key_name=key_with_rightbrace_name)
        key_with_rightbrace.set_contents_from_string('key with right brace')

        ### Get keys and check content
        key_with_leftbrace = self.bucket_obj.get_key(key_name=key_with_leftbrace_name)
        self.assertEqual(key_with_leftbrace.read(), 'key with left brace')

        key_with_rightbrace = self.bucket_obj.get_key(key_name=key_with_rightbrace_name)
        self.assertEqual(key_with_rightbrace.read(), 'key with right brace')

        ### Delete keys
        self.bucket_obj.delete_key(key_name=key_with_leftbrace_name)
        self.bucket_obj.delete_key(key_name=key_with_rightbrace_name)

    def test_all(self):
        ### Create keys and set content
        key_with_star = self.bucket_obj.new_key(key_name=key_with_star_name)
        key_with_star.set_contents_from_string('key with star')

        key_with_leftbrace = self.bucket_obj.new_key(key_name=key_with_leftbrace_name)
        key_with_leftbrace.set_contents_from_string('key with left brace')

        key_with_dollar = self.bucket_obj.new_key(key_name=key_with_dollar_name)
        key_with_dollar.set_contents_from_string('key with dollar')

        key_with_exclamation = self.bucket_obj.new_key(key_name=key_with_exclamation_name)
        key_with_exclamation.set_contents_from_string('key with exclamation')

        key_with_backtick = self.bucket_obj.new_key(key_name=key_with_backtick_name)
        key_with_backtick.set_contents_from_string('key with backtick')

        key_with_atrate = self.bucket_obj.new_key(key_name=key_with_atrate_name)
        key_with_atrate.set_contents_from_string('key with at rate')

        key_with_ampersand = self.bucket_obj.new_key(key_name=key_with_ampersand_name)
        key_with_ampersand.set_contents_from_string('key with ampersand')

        key_with_rightbrace = self.bucket_obj.new_key(key_name=key_with_rightbrace_name)
        key_with_rightbrace.set_contents_from_string('key with right brace')

        key_with_pound = self.bucket_obj.new_key(key_name=key_with_pound_name)
        key_with_pound.set_contents_from_string('key with pound')

        key_with_caret = self.bucket_obj.new_key(key_name=key_with_caret_name)
        key_with_caret.set_contents_from_string('key with caret')

        key_with_percent = self.bucket_obj.new_key(key_name=key_with_percent_name)
        key_with_percent.set_contents_from_string('key with percent')

        key_with_underscore = self.bucket_obj.new_key(key_name=key_with_underscore_name)
        key_with_underscore.set_contents_from_string('key with underscore')

        ### Get keys and check content
        key_with_star = self.bucket_obj.get_key(key_name=key_with_star_name)
        self.assertEqual(key_with_star.read(), 'key with star')

        key_with_dollar = self.bucket_obj.get_key(key_name=key_with_dollar_name)
        self.assertEqual(key_with_dollar.read(), 'key with dollar')

        key_with_exclamation = self.bucket_obj.get_key(key_name=key_with_exclamation_name)
        self.assertEqual(key_with_exclamation.read(), 'key with exclamation')

        key_with_percent = self.bucket_obj.get_key(key_name=key_with_percent_name)
        self.assertEqual(key_with_percent.read(), 'key with percent')

        key_with_underscore = self.bucket_obj.get_key(key_name=key_with_underscore_name)
        self.assertEqual(key_with_underscore.read(), 'key with underscore')

        key_with_backtick = self.bucket_obj.get_key(key_name=key_with_backtick_name)
        self.assertEqual(key_with_backtick.read(), 'key with backtick')

        key_with_atrate = self.bucket_obj.get_key(key_name=key_with_atrate_name)
        self.assertEqual(key_with_atrate.read(), 'key with at rate')

        key_with_pound = self.bucket_obj.get_key(key_name=key_with_pound_name)
        self.assertEqual(key_with_pound.read(), 'key with pound')

        key_with_caret = self.bucket_obj.get_key(key_name=key_with_caret_name)
        self.assertEqual(key_with_caret.read(), 'key with caret')

        key_with_ampersand = self.bucket_obj.get_key(key_name=key_with_ampersand_name)
        self.assertEqual(key_with_ampersand.read(), 'key with ampersand')

        key_with_leftbrace = self.bucket_obj.get_key(key_name=key_with_leftbrace_name)
        self.assertEqual(key_with_leftbrace.read(), 'key with left brace')

        key_with_rightbrace = self.bucket_obj.get_key(key_name=key_with_rightbrace_name)
        self.assertEqual(key_with_rightbrace.read(), 'key with right brace')

        ### Delete keys
        self.bucket_obj.delete_key(key_name=key_with_star_name)
        self.bucket_obj.delete_key(key_name=key_with_dollar_name)
        self.bucket_obj.delete_key(key_name=key_with_exclamation_name)
        self.bucket_obj.delete_key(key_name=key_with_percent_name)
        self.bucket_obj.delete_key(key_name=key_with_underscore_name)
        self.bucket_obj.delete_key(key_name=key_with_backtick_name)
        self.bucket_obj.delete_key(key_name=key_with_atrate_name)
        self.bucket_obj.delete_key(key_name=key_with_pound_name)
        self.bucket_obj.delete_key(key_name=key_with_caret_name)
        self.bucket_obj.delete_key(key_name=key_with_ampersand_name)
        self.bucket_obj.delete_key(key_name=key_with_leftbrace_name)
        self.bucket_obj.delete_key(key_name=key_with_rightbrace_name)

    def tearDown(self):
        ### Tear down the bucket
        self.tester.delete_bucket(bucket=self.bucket_obj)
        self.tester=None

if __name__ == "__main__":
    ### Parse command line arguments and get the path to credentials
    parser = argparse.ArgumentParser(description='Unit tests to verify: fix for EUCA-3453, and handling of special '
                                                 'characters in Walrus object names')
    parser.add_argument('--credpath', dest='credpath', required=True, help='Path to folder containing credentials')
    args = parser.parse_args()

    try:
        ### Setup the cloud connection
        eucaops = Eucaops(credpath=args.credpath)
    except Exception, e:
        exit('Failed to establish connection to cloud due to: ' + str(e))

    ### Run the unit test
    suite = unittest.TestLoader().loadTestsFromTestCase(EUCA3453)
    unittest.TextTestRunner(verbosity=2).run(suite)

    eucaops=None



# Software License Agreement (BSD License)
#
# Copyright (c) 2009-2011, Eucalyptus Systems, Inc.
# All rights reserved.
#
# Redistribution and use of this software in source and binary forms, with or
# without modification, are permitted provided that the following conditions
# are met:
#
#   Redistributions of source code must retain the above
#   copyright notice, this list of conditions and the
#   following disclaimer.
#
#   Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the
#   following disclaimer in the documentation and/or other
#   materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# Author: vic.iglesias@eucalyptus.com
import select
import threading
import time
import eulogger
from eutester import Eutester
from eutester.euconfig import EuConfig
import sshconnection
import re
import os
import sys
import tempfile
from repoutils import RepoUtils

class DistroName:
    ubuntu = "ubuntu"
    rhel = "rhel"
    centos = "centos"
    fedora = "fedora"
    debian = "debian"
    vmware = "vmware"

class DistroRelease:
    def __init__(self, distro_name, distro_number,  distro_release = "", package_manager= None):
        self.name = distro_name
        self.number = distro_number
        self.release = distro_release
        self.package_manager = package_manager
        
class Distro:
    ubuntu_lucid = DistroRelease(DistroName.ubuntu,"10.04",  "lucid",  package_manager="apt")
    ubuntu_precise = DistroRelease(DistroName.ubuntu,  "12.04", "precise",  package_manager="apt")
    debian_squeeze = DistroRelease(DistroName.debian, "6", "squeeze",  package_manager="apt")
    debian_wheezy = DistroRelease(DistroName.debian, "7", "wheezy",  package_manager="apt")
    rhel_6 = DistroRelease(DistroName.rhel, "6",  package_manager="yum")
    centos_6 = DistroRelease(DistroName.centos, "6",  package_manager="yum")
    rhel_5 = DistroRelease(DistroName.rhel, "5",  package_manager="yum")
    centos_5 = DistroRelease(DistroName.centos, "5",  package_manager="yum")
    fedora_18 = DistroRelease(DistroName.fedora, "18",  package_manager="yum")
    vmware_5 = DistroRelease(DistroName.vmware, "5")
    vmware_4 = DistroRelease(DistroName.vmware, "4")

    @classmethod
    def get_distros(Distro):
        distros = []
        for distro in Distro.__dict__:
            if isinstance(Distro.__dict__[distro], DistroRelease):
                distros.append(Distro.__dict__[distro])
        return distros
        
    

class Machine:
    def __init__(self, 
                 hostname, 
                 distro="", 
                 distro_ver="", 
                 arch="", 
                 source="", 
                 components="", 
                 connect=True, 
                 password=None, 
                 keypath=None, 
                 username="root", 
                 timeout=120,
                 retry=2,
                 debugmethod=None, 
                 verbose = True ):
        
        self.hostname = hostname
        self.distro_ver = distro_ver
        self.distro = self.convert_to_distro(distro, distro_ver)
        if self.distro.package_manager is not None:
            self.repo_utils = RepoUtils(self, self.distro.package_manager)
            self.package_manager = self.repo_utils.package_manager
        self.arch = arch
        self.source = source
        self.components = components
        self.connect = connect
        self.password = password
        self.keypath = keypath
        self.username = username
        self.timeout = timeout
        self.retry = retry
        self.debugmethod = debugmethod
        self.verbose = verbose
        self.log_threads = {}
        self.log_buffers = {}
        self.log_active = {}
        self.wget_last_status = 0
        if self.debugmethod is None:
            logger = eulogger.Eulogger(identifier= str(hostname) + ":" + str(components))
            self.debugmethod = logger.log.debug
        if self.connect:
            self.ssh = sshconnection.SshConnection( hostname, 
                                                    keypath=keypath,          
                                                    password=password, 
                                                    username=username, 
                                                    timeout=timeout, 
                                                    retry=retry,
                                                    debugmethod=self.debugmethod,
                                                    verbose=True)
            self.sftp = self.ssh.connection.open_sftp()
        # If we were given a conf file, and have an ssh/sftp session, attempt to populate eucalyptus_conf into
        # a euconfig object for this machine...
        self.get_eucalyptus_conf()
            
    def convert_to_distro(self, distro_name, distro_release):
        distro_name = distro_name.lower()
        distro_release = distro_release.lower()
        for distro in Distro.get_distros():
            if re.search( distro.name, distro_name,re.IGNORECASE) and \
                    (re.search( distro.release, distro_release,re.IGNORECASE) or
                         re.search( distro.number, distro_release,re.IGNORECASE)) :
                return distro
        raise Exception("Unable to find distro " + str(distro_name) + " and version "
                        + str(distro_release) + " for hostname " + str(self.hostname))

    def put_templated_file(self, local_src, remote_dest, **kwargs):
        tmp = tempfile.mktemp()
        try:
            Eutester.render_file_template(local_src, tmp, **kwargs)
            self.ssh.sftp_put(tmp, remote_dest)
        finally:
            os.remove(tmp)

    def refresh_ssh(self):
        self.ssh.refresh_connection()
        
    def debug(self,msg):
        """
        Used to print debug, defaults to print() but over ridden by self.debugmethod if not None
        msg - mandatory -string, message to be printed
        """
        if self.verbose is True:
                self.debugmethod(msg)
                
    def refresh_connection(self):
        self.ssh.refresh_connection()

    def reboot(self, force=True):
        if force:
            try:
                self.sys("reboot -f", timeout=3)
            except Exception, e:
                pass
        else:
            try:
                self.sys("reboot", timeout=3)
            except Exception, e:
                pass
    
    def interrupt_network(self, time = 120, interface = "eth0"):
        try:
            self.sys("ifdown " + interface + " && sleep " + str(time) + " && ifup " + interface,  timeout=3)
        except Exception,e:
            pass

    def sys(self, cmd, verbose=True, timeout=120, listformat=True, code=None):
        '''
        Issues a command against the ssh connection to this instance
        Returns a list of the lines from stdout+stderr as a result of the command
        '''
        return self.ssh.sys(cmd, verbose=verbose, timeout=timeout,listformat=listformat, code=code)
    
    def cmd(self, cmd, verbose=True, timeout=120, listformat=False, cb=None, cbargs=[]):
        '''
        Issues a command against the ssh connection to this instance
        returns dict containing:
            ['cmd'] - The command which was executed
            ['output'] - The std out/err from the executed command
            ['status'] - The exit (exitcode) of the command, in the case a call back fires, this status code is unreliable.
            ['cbfired']  - Boolean to indicate whether or not the provided callback fired (ie returned False)
            ['elapsed'] - Time elapsed waiting for command loop to end. 
        cmd - mandatory - string, the command to be executed 
        verbose - optional - boolean flag to enable debug
        timeout - optional - command timeout in seconds 
        listformat -optional - specifies returned output in list of lines, or single string buffer
        cb - optional - call back function, accepting string buffer, returning true false see sshconnection for more info
        '''
        if (self.ssh is not None):
            return self.ssh.cmd(cmd, verbose=verbose, timeout=timeout, listformat=listformat, cb=cb, cbargs=cbargs)
        else:
            raise Exception("Euinstance ssh connection is None")
        
    def sys_until_found(self, cmd, regex, verbose=True, timeout=120, listformat=True):
        '''
        Run a command until output of command satisfies/finds regex or EOF is found. 
        returns dict containing:
            ['cmd'] - The command which was executed
            ['output'] - The std out/err from the executed command
            ['status'] - The exit (exitcode) of the command, in the case a call back fires, this status code is unreliable.
            ['cbfired']  - Boolean to indicate whether or not the provided callback fired (ie returned False)
            ['elapsed'] - Time elapsed waiting for command loop to end.
        cmd - mandatory - string, the command to be executed 
        regex - mandatory - regex to look for
        verbose - optional - boolean flag to enable debug
        timeout - optional - command timeout in seconds 
        listformat -optional - specifies returned output in list of lines, or single string buffer 
        '''
        return self.cmd(cmd, verbose=verbose,timeout=timeout,listformat=listformat,cb=self.str_found_cb, cbargs=[regex, verbose])
        
        
    def str_found_cb(self,buf,regex,verbose,search=True):
        '''
        Return sshcbreturn type setting stop to True if given regex matches against given string buf
        '''
        if verbose:
            self.debug(str(buf))
        return sshconnection.SshCbReturn( stop=self.str_found(buf, regex=regex, search=search))
        
        
    def str_found(self, buf, regex, search=True):
        '''
        Return True if given regex matches against given string
        '''
        if search:
            found = re.search(regex,buf)
        else:
            found = re.match(regex, buf)
        if found:
            return True
        else:
            return False

    def get_eucalyptus_service_pid(self, eucalyptus_service):
        """
        Returns the process id or pid of the eucalyptus service running on this machine. Will return None if not found,
        which may indicate the process is not running or not intended to run on this machine.

        :param eucalyptus_service: eucalyptus-cloud, eucalyptus-cc, eucalyptus-nc
        :return: string representing pid
        """
        pid = None
        paths = ["/var/run/eucalyptus/","/opt/eucalyptus/var/run/eucalyptus/"]
        for path in paths:
            try:
                pid = int(self.sys('cat ' + path + str(eucalyptus_service), code=0)[0].strip())
                break
            except: pass
        if pid is None:
            self.debug("Pid not found at paths: ".join(paths))
        return pid

    def get_eucalyptus_cloud_pid(self):
        """
        :return: Returns the process id for eucalyptus-cloud running on this machine, or None if not found.
        """
        return self.get_eucalyptus_service_pid('eucalyptus-cloud.pid')

    def get_eucalyptus_nc_pid(self):
        """
        :return: Returns the process id for eucalyptus-nc running on this machine, or None if not found.
        """
        return self.get_eucalyptus_service_pid('eucalyptus-nc.pid')

    def get_eucalyptus_cc_pid(self):
        """
        :return: Returns the process id for eucalyptus-cc running on this machine, or None if not found.
        """
        return self.get_eucalyptus_service_pid('eucalyptus-cc.pid')

    def get_uptime(self):
        return int(self.sys('cat /proc/uptime', code=0)[0].split()[1].split('.')[0])

    def get_eucalyptus_cloud_process_uptime(self):
        """
        Attempts to look up the elapsed running time of the PID associated with the eucalyptus-cloud process/service.
        :return: (int) elapsed time in seconds this PID has been running
        """
        pid = self.get_eucalyptus_cloud_pid()
        return self.get_elapsed_seconds_since_pid_started(pid)

    def get_eucalyptus_cc_process_uptime(self):
        """
        Attempts to look up the elapsed running time of the PID associated with the eucalyptus-cc process/service.
        :return: (int) elapsed time in seconds this PID has been running
        """
        pid = self.get_eucalyptus_cc_pid()
        return self.get_elapsed_seconds_since_pid_started(pid)

    def get_eucalyptus_nc_process_uptime(self):
        """
        Attempts to look up the elapsed running time of the PID associated with the eucalyptus-nc process/service.
        :return: (int) elapsed time in seconds this PID has been running
        """
        pid = self.get_eucalyptus_nc_pid()
        return self.get_elapsed_seconds_since_pid_started(pid)

    def get_eucalyptus_cloud_is_running_status(self):
        """
        Checks eucalyptus-cloud service status
        :return: boolean, True if running False if not.
        """
        return self.get_service_is_running_status('eucalyptus-cloud')

    def get_eucalyptus_cc_is_running_status(self):
        """
        Checks eucalyptus-cc service status
        :return: boolean, True if running False if not.
        """
        return self.get_service_is_running_status('eucalyptus-cc')

    def get_eucalyptus_nc_is_running_status(self):
        """
        Checks eucalyptus-nc service status
        :return: boolean, True if running False if not.
        """
        return self.get_service_is_running_status('eucalyptus-nc')


    def get_service_is_running_status(self, service, code=0):
        """
        Checks status of service 'service' on the machine obj.
        :param service: string representing service name
        :return: boolean.
        """
        try:
            self.sys("service " + str(service) + " status", code=0)
            return True
        except sshconnection.CommandExitCodeException:
            return False
        except Exception, e:
            self.debug('Could not get "'+ str(service) + '" service state from machine:'
                       + str(self.hostname) + ", err:"+str(e))

    def get_elapsed_seconds_since_pid_started(self, pid):
        """
        Attempts to parse ps time elapsed since process/pid has been running and return the presented time in
        elapsed number of seconds.
        :param pid: Process id to get elapsed time from
        :return: Elapsed time in seconds that pid has been running
        """
        seconds_min = 60
        seconds_hour = 3600
        seconds_day = 86400
        elapsed = 0
        try:
            if not pid:
                raise Exception('Empty pid passed to get_elapsed_seconds_since_pid_started')
            cmd = "ps -eo pid,etime | grep " + str(pid) + " | awk '{print $2}'"
            self.debug('starting get pid uptime"' + str(cmd) + '"...')
            #expected format: days-HH:MM:SS
            out = self.sys(cmd,code=0)[0]
            out = out.strip()
            if re.search("-", out):
                split_out = out.split("-")
                days =  int(split_out[0])
                time_string = split_out[1]
            else:
                days = 0
                time_string = out

            split_time = time_string.split(':')
            #insert a 0 if hours, and minutes are not present.
            for x in xrange(len(split_time), 3):
                split_time.insert(0,0)

            hours = int(split_time[0] or 0)
            minutes = int(split_time[1] or 0)
            seconds = int(split_time[2] or 0)
            elapsed = seconds + (minutes*seconds_min) + (hours*seconds_hour) + (days*seconds_day)
        except:
            print Eutester.get_traceback()
        return int(elapsed)

    def get_eucalyptus_version(self,versionpath="/etc/eucalyptus/eucalyptus-version"):
        """

        :param versionpath: path to version file
        :return: eucalyptus version string
        """
        try:
            return self.sys('cat ' + versionpath, code=0)[0]
        except Exception, e:
            return self.sys('cat /opt/eucalyptus' + versionpath, code=0)[0]



    def is_file_present(self, filepath):
        try:
            self.get_file_stat(filepath)
        except IOError, io:
            #IOError: [Errno 2] No such file
            if io.errno == 2:
                return False
            else:
                raise io
        return True


    def get_file_stat(self,path):
        return self.sftp.lstat(path)
        
    def get_file_size(self, path):
        return self.sftp.lstat(path).st_size
    
    def get_file_perms_flag(self,path):
        return self.sftp.lstat(path).FLAG_PERMISSIONS 
    
    def get_file_groupid(self, path):
        return self.sftp.lstat(path).st_gid
        
    def get_file_userid(self,path):
        return self.sftp.lstat(path).st_uid
    
    def get_masked_pass(self, pwd, firstlast=True, charcount=True, show=False):
        '''
        format password for printing
        options:
        pwd - string- the text password to format
        firstlast -boolean - show the first and last characters in pwd
        charcount -boolean - print a "*" for each char in pwd, otherwise return fixed string '**hidden**'
        show - boolean - convert pwd to str() and return it in plain text
        '''
        ret =""
        if pwd is None:
            return ""
        if show is True:
            return str(pwd)
        if charcount is False:
            return "**hidden**"
        for x in xrange(0,len(pwd)):
            if (x == 0 or x == len(pwd)) and firstlast:
                ret = ret+pwd[x]
            else:
                ret += "*"

    def mkfs(self, partition, type="ext3"):
        self.sys("mkfs."+ type + " -F " + partition)

    def mount(self, device, path):
        self.sys("mount "+ device + " " + path)

    def chown(self, user, path):
        self.sys("chwon "+ user + ":" + user + " " + path)

    def ping_check(self,host):
        out = self.ping_cmd(host)
        self.debug('Ping attempt to host:'+str(host)+", status code:"+str(out['status']))
        if out['status'] != 0:
            raise Exception('Ping returned error:'+str(out['status'])+' to host:'+str(host))
    
    def ping_cmd(self, host, count=2, pingtimeout=10, commandtimeout=120, listformat=False, verbose=True):
        cmd = 'ping -c ' +str(count)+' -t '+str(pingtimeout)
        if verbose:
            cmd += ' -v '
        cmd = cmd + ' '+ str(host)
        out = self.cmd(cmd, verbose=verbose, timeout=commandtimeout, listformat=listformat)
        if verbose:
            #print all returned attributes from ping command dict
            for item in sorted(out):
                self.debug(str(item)+" = "+str(out[item]) )  
        return out
        
        
    def dump_netfail_info(self,ip=None, mac=None, pass1=None, pass2=None, showpass=True, taillength=50):
        """
        Debug method to provide potentially helpful info from current machine when debugging connectivity issues.
        """
        self.debug('Attempting to dump network information, args: ip:' + str(ip)
                   + ' mac:' + str(mac)
                   + ' pass1:' + self.get_masked_pass(pass1,show=True)
                   + ' pass2:' + self.get_masked_pass(pass2,show=True))
        self.ping_cmd(ip,verbose=True)
        self.sys('arp -a')
        self.sys('dmesg | tail -'+str(taillength))
        self.sys('cat /var/log/messages | tail -'+str(taillength))
        
    def found(self, command, regex, verbose=True):
        """ Returns a Boolean of whether the result of the command contains the regex"""
        result = self.sys(command, verbose=verbose)
        if result is None or result == []:
            return False
        for line in result:
            found = re.search(regex,line)
            if found:
                return True
        return False   
    
    def wget_remote_image(self, url,path=None, user=None, password=None, retryconn=True, timeout=300):
        self.debug('wget_remote_image, url:'+str(url)+", path:"+str(path))
        cmd = 'wget '
        if path:
            cmd = cmd + " -P " + str(path)
        if user:
            cmd = cmd + " --user " + str(user)
        if password:
            cmd = cmd + " --password " + str(password)
        if retryconn:
            cmd += ' --retry-connrefused '
        cmd = cmd + ' ' + str(url)
        self.debug('wget_remote_image cmd: '+str(cmd))
        ret = self.cmd(cmd, timeout=timeout, cb=self.wget_status_cb )
        if ret['status'] != 0:
            raise Exception('wget_remote_image failed with status:'+str(ret['status']))
        self.debug('wget_remote_image succeeded')
    
    def wget_status_cb(self, buf):
        ret = sshconnection.SshCbReturn(stop=False)
        try:
            buf = buf.strip()
            val = buf.split()[0] 
            if val != self.wget_last_status:
                if re.match('^\d+\%',buf):
                    sys.stdout.write("\r\x1b[K"+str(buf))
                    sys.stdout.flush()
                    self.wget_last_status = val
                else:
                    print buf
        except Exception, e:
            pass
        finally:
            return ret

            
        
    def get_df_info(self, path=None, verbose=True):
        """
        Return df's output in dict format for a given path.
        If path is not given will give the df info for the current working dir used in the ssh
        session this command is executed in (ie: /home/user or /root).
        path - optional -string, used to specifiy path to use in df command. Default is PWD of ssh shelled command
        verbose - optional -boolean, used to specify whether or debug is printed during this command.
        Example:
            dirpath = '/disk1/storage'
            dfout = self.get_df_info(path=dirpath)
            available_space = dfout['available']
            mounted_on = dfout['mounted']
            filesystem = dfout['filesystem']
        """
        ret = {}
        if path is None:
            path = '${PWD}'
        cmd = 'df '+str(path)
        if verbose:
            self.debug('get_df_info cmd:'+str(cmd))
        output = self.sys(cmd, code=0)
        # Get the presented fields from commands output,
        # Convert to lowercase, use this as our dict keys
        fields=[]
        line = 0
        for field in str(output[line]).split():
            fields.append(str(field).lower())
        # Move line forward and gather columns into the dict to be returned
        x = 0 
        line += 1
        # gather columns equal to the number of column headers accounting for newlines...
        while x < (len(fields)-1):
            for value in str(output[line]).split():
                ret[fields[x]]=value
                if verbose:
                    self.debug(str('DF FIELD: '+fields[x])+' = '+str(value))
                x += 1
            line += 1
        return ret
    
    def upgrade(self, package=None, nogpg=False):
        self.package_manager.upgrade(package, nogpg=nogpg)
    
    def add_repo(self, url, name="test-repo"):
        self.package_manager.add_repo(url,name)
    
    def install(self, package, nogpg=False):
        self.package_manager.install(package,nogpg=nogpg)

    def update_repos(self):
        self.package_manager.update_repos()
    
    def get_package_info(self):
        self.package_manager.get_package_info()
    
    def get_installed_packages(self):
        self.package_manager.get_installed_packages()
            
    def get_available(self, path, unit=1):
        """
        Return df output's available field. By default this is KB.
        path - optional -string.
        unit - optional -integer used to divide return value. Can be used to convert KB to MB, GB, TB, etc..
        """
        size = int(self.get_df_info(path=path)['available'])
        return size/unit
    
    def poll_log(self, log_file="/var/log/messages"):
        self.debug( "Starting to poll " + log_file )     
        self.log_channel = self.ssh.connection.invoke_shell()
        self.log_channel.send("tail -f " + log_file + " \n")
        ### Begin polling channel for any new data
        while self.log_active[log_file]:
            ### CLOUD LOG
            rl, wl, xl = select.select([self.log_channel],[],[],0.0)
            if len(rl) > 0:
                self.log_buffers[log_file] += self.log_channel.recv(1024)
            time.sleep(1)
    
    def start_log(self, log_file="/var/log/messages"):
        """Start thread to poll logs"""
        thread = threading.Thread(target=self.poll_log, args=log_file)
        thread.daemon = True
        self.log_threads[log_file]= thread.start()
        self.log_active[log_file] = True
        
    def stop_log(self, log_file="/var/log/messages"):
        """Terminate thread that is polling logs"""
        self.log_active[log_file] = False
        
    def save_log(self, log_file, path="logs"):
        """Save log buffer for log_file to the path to a file"""
        if not os.path.exists(path):
            os.mkdir(path)
        FILE = open( path + '/' + log_file,"w")
        FILE.writelines(self.log_buffers[log_file])
        FILE.close()
        
    def save_all_logs(self, path="logs"):
        """Save log buffers to a file"""
        for log_file in self.log_buffers.keys():
            self.save_log(log_file,path)

    def get_eucalyptus_conf(self,eof=False,verbose=False):
        out = None
        config = None
        use_path = None
        paths = [ "/" , "/opt/eucalyptus" ]
        for path in paths:
            try:
                self.sys('ls '+ str(path) + '/etc/eucalyptus/eucalyptus.conf', code=0, verbose=verbose)
                use_path = path + '/etc/eucalyptus/eucalyptus.conf'
                break
            except:
                pass
        if not use_path:
            out = 'eucalyptus.conf not found on this system'
            if eof:
                raise Exception(eof)
            else:
                self.debug(out)
        else:
            try:
                config = EuConfig(filename=use_path, ssh=self.ssh, default_section_name='eucalyptus_conf')
                self.config = config
                if hasattr(config, 'eucalyptus_conf'):
                    self.eucalyptus_conf = config.eucalyptus_conf
            except Exception, e:
                out = 'Error while trying to create euconfig from eucalyptus_conf:' + str(e)
                if eof:
                    raise Exception(out)
                else:
                    self.debug(out)
        return config

            
        
    
    def __str__(self):
        s  = "+++++++++++++++++++++++++++++++++++++++++++++++++++++\n"
        s += "+" + "Hostname:" + str(self.hostname) + "\n"
        dname = self.distro.name if self.distro else ""
        s += "+" + "Distro: " + str(dname) +"\n"
        s += "+" + "Distro Version: " +  str(self.distro_ver) +"\n"
        s += "+" + "Install Type: " +  str(self.source) +"\n"
        s += "+" + "Components: " +   str(self.components) +"\n"
        s += "+++++++++++++++++++++++++++++++++++++++++++++++++++++"
        return s
